from pathlib import Path

from spicy_snow.retrieval import retrieve_snow_depth
from spicy_snow.IO.user_dates import get_input_dates

import shapely

# change to your minimum longitude, min lat, max long, max lat
area = shapely.geometry.box(8, 45.5, 12, 48)

# this will be where your results are saved
out_nc = Path('./switzerland.nc').expanduser()

# this will generate a tuple of dates from the previous August 1st to this date
dates = get_input_dates('2018-03-01') # run on all s1 images from (2020-08-01, 2021-04-01) in this example

spicy_ds = retrieve_snow_depth(area = area, dates = dates, 
                               work_dir = Path('~/scratch/switzerland').expanduser(), 
                               job_name = f'switerzerland',
                               existing_job_name = 'switerzerland',
                               debug=False,
                               outfp=out_nc,
                               params = [2, 0.5, 0.44])


from pathlib import Path

from spicy_snow.download.snowex_lidar import download_dem, download_snow_depth,\
      download_veg_height, make_site_ds

lidar_dir = Path('/bsuhome/zacharykeskinen/scratch/spicy/lidar')
lidar_dir.mkdir(exist_ok = True)
download_snow_depth(lidar_dir)
download_veg_height(lidar_dir)
download_dem(lidar_dir)



# produce 202011 to 202104 statistic with different zonal, time range, time close, precipitation, and method conditions,
# and compare it with NASA snowEx observation data.

import os

from asf_snow.utils.combine_multiple_statistic_results import combine_multiple_statistic_results

from asf_snow.utils.compare_sentinel1_snow_depth_to_snex import *

from asf_snow.utils.analyze_sentinel1_snow import draw_combined_gdf, draw_scatter_plot

os.chdir('/home/jiangzhu/projects/work/crrel/asf-snow/src/asf_snow/utils')

varname = 'snow_depth'
location = 'Boise'
res = 500
timeint = 36
timethresh = 6
base = 'snex'
method = "close"
precip = 'y'
snexfile = '/home/jiangzhu/data/crrel/snowEx21_ts_snow_pits_v1/SNEX21_TS_SP_Summary_SWE_v01.csv'

ncfilelst = [
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20201101_1130/sentinel1_20201101_1130.nc',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20201201_1231/sentinel1_20201201_1231.nc',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210101_0131/sentinel1_20210101_0131.nc',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210201_0228/sentinel1_20210201_0228.nc',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210301_0331/sentinel1_20210301_0331.nc',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210401_0430/sentinel1_20210401_0430.nc',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210501_0531/sentinel1_20210501_0531.nc',
]

# list of the statistic file for every month
outfilelst = []

for ncfile in ncfilelst:

    file = compare_sentinel1_snow_depth_to_snex(ncfile, varname, snexfile, location, res, timeint,
                                                timethresh, base, method, precip)
    outfilelst.append(file)

# write the outfilelst

out_dir = str(Path(file).parent.parent)

affix = f'_{int(res)}_{timeint}h_{timethresh}h_{base}_{method}_{precip}'

outfilelist = f'{out_dir}/boise_202011_202104{affix}.txt'

with open(outfilelist, 'w') as f:
    for line in outfilelst:
        f.write(f"{line}\n")

# combine the statistic files in the outfilelst

combined = combine_multiple_statistic_results(outfilelst)

# draw combined

outgeojson = f'{out_dir}/sentinel1_202011_202104_{res}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_boise_river_basin.geojson'

outpngfile = f'{out_dir}/sentinel1_202011_202104_{res}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_boise_river_basin.png'

if combined is not None:

    if not combined.empty:

        combined.to_file(outgeojson, driver='GeoJSON')

        draw_combined_gdf(combined, res=res, timeint=timeint, timethresh=timethresh, base=base, method=method,
                          outfile=outpngfile, precip=precip)

print("completed ...")



from pathlib import Path
import sys
import glob
from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser

from asf_snow.utils.analyze_sentinel1_snow import *

from asf_snow.utils.analyze_snex import *

# compare the snow depth derived with sentinel1 to NASA SnowEx experiment
# example arguments
# ncfile = "/home/jiangzhu/data/crrel/sentinel1_snowEx21_ts_snow_pits_v1/sentinel1.nc"
# snexfile = "/home/jiangzhu/data/crrel/snowEx21_ts_snow_pits_v1/302507681/SNEX21_TS_SP_Summary_SWE_v01.csv"
# snexenvfile = "/home/jiangzhu/data/crrel/snowEx21_ts_snow_pits_v1/SNEX21_TS_SP_Summary_Environment_v01.csv"
# varname = "snow_depth"
# res = 1000  # zonal analysis square size


# step 1. download SNOTEL data


# Step 2. read the 

# lon = -147.733, lat =  64.867
def compare_sentinel1_snow_depth_to_snotel_dry_wet(ncfile, varname, snotelfile, lon, lat, res, timeint, timethresh, base, method,
                                         precip):

    # sentinel1 snow depth data
    # nc_var = get_var(ncfile, varname)
    nc_var = get_ds(ncfile)
    #if varname in ['deltaCRt']:

    # convert data in db to ratio
    nc_var.deltaCRt.data = 10.0**nc_var.deltaCRt.data

    # SNOTEL snow depth data

    #test dry/wet snow
    # snotel = read_snotel(snotelfile, lon, lat)

    snotel = read_snotel_sd_precip_ratio_temp(snotelfile, lon, lat, {'Location':'creamers'})

    ch = (snotel['precip_inc'] > 0.0) & (snotel['snow_rain_ratio'] > 50.0)

    snotel_wet = snotel[ch]

    snotel_dry = snotel[~ch]

    if precip == 'y':
        snotel = snotel_wet
    else:
        snotel = snotel_dry

    if base == 'snotel':
        # call zonal_analyses_based_snex
        combined = zonal_analyses_based_snex(nc_var, snotel, res=res, timeint=timeint, timethresh=timethresh,
                                             method=method)
    else:
        # call zonal_analyses_based_sentinel1
        combined = zonal_analyses_s1_snotel_dry_wet_based_s1(nc_var, snotel, res=res, timeint=timeint,
                                                  timethresh=timethresh, method=method)

    loc = snotel.attrs['Location']
    affix = f'_{int(res)}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_{loc.replace(" ", "_")}_{varname.lower()}.shp'.lower()

    affix_geojson = f'_{int(res)}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_{loc.replace(" ", "_")}_{varname.lower()}.geojson'.lower()

    outfile = ncfile.replace(".nc", affix)

    outfile_geojson = ncfile.replace(".nc", affix_geojson)

    pwildcard = str(Path(outfile).parent.joinpath(Path(outfile).stem)) + ".*"

    for file in glob.glob(pwildcard):
        # remove the file if it exists
        Path(file).unlink(missing_ok=True)

    # if not combined.empty:
    if type(combined) is not None:
        if not combined.empty:
            # save the combined GeoDataFrame
            # combined.to_file(outfile)
            combined.to_file(outfile_geojson, driver='GeoJSON')
        # break
        # draw line plot
        # pngfile = outfile.replace('.shp', '.png')
        # draw_combined_gdf(combined, res=res, timeint=timeint, timethresh=timethresh, flg=method,
        #                  outfile=pngfile, precip=precip)

    return outfile_geojson


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--ncfile', type=str,  required=True)
    parser.add_argument('--varname', type=str, default='snow_depth')
    parser.add_argument('--snotelfile', type=str,  required=True)
    parser.add_argument('--lon', type=float )
    parser.add_argument('--lat', type=float)
    parser.add_argument('--res', type=float, default=100)
    parser.add_argument('--timeint', type=int, default=24)
    parser.add_argument('--timethresh', type=int, default=6)
    parser.add_argument('--base', choices=['snotel', 's1'], default='s1')
    parser.add_argument('--method', choices=['nearest', 'average', 'close'], default='average')
    parser.add_argument('--precip', choices=['y', 'n'], default='y')

    args = parser.parse_args()

    compare_sentinel1_snow_depth_to_snotel_dry_wet(args.ncfile, varname=args.varname, snotelfile=args.snotelfile, lon=args.lon,
                                           lat=args.lat, res=args.res, timeint=args.timeint,
                                           timethresh=args.timethresh, base=args.base, method=args.method,
                                         precip=args.precip)



    print('completed analysis...')



import sys
from asf_snow.utils.analyze_sentinel1_snow import *
from asf_snow.utils.analyze_snex import *
from asf_snow.utils.utils import write_polygon
from shapely import geometry, to_geojson

import json


def readgeojson(geojsfile):

    f = open(geojsfile)

    jsstr = json.load(f)

    return geometry.shape(jsstr)

def draw_multiple_lines(merged, loc, outfile):
    # merged = pd.merge_asof(s1_df, snotel, on="time", direction="nearest")
    # when calculating corr, get rid of NAN data

    merged1 = merged[~merged['snow_depth_y'].isnull()]

    # claculate the statistic
    corr = merged1['snow_depth_y'].corr(merged1['snow_depth_x'])
    corr = float("{:.2f}".format(corr))
    mean = (merged1['snow_depth_y'] - merged1['snow_depth_x']).mean()
    mean = float("{:.2f}".format(mean))
    std = (merged1['snow_depth_y'] - merged1['snow_depth_x']).std()
    std = float("{:.2f}".format(std))
    min = (merged1['snow_depth_y'] - merged1['snow_depth_x']).min()
    min = float("{:.2f}".format(min))
    max = (merged1['snow_depth_y'] - merged1['snow_depth_x']).max()
    max = float("{:.2f}".format(max))

    st = pd.to_datetime(merged.Time_x.iloc[0])
    st_date = st.strftime('%Y-%m-%d')
    ed = pd.to_datetime(merged.Time_x.iloc[-1])
    ed_date = ed.strftime('%Y-%m-%d')

    # set global parameters for the figure
    params = {'figure.figsize': (30, 30),
              'legend.fontsize': 20, 'legend.handlelength': 2,
              'axes.titlesize': 22,
              'axes.labelsize': 20,
              'xtick.labelsize': 20,
              'ytick.labelsize': 20
              }
    plt.rcParams.update(params)

    if 'snow_rain_ratio' in merged.columns:
        fig, ax = plt.subplots(nrows=5, ncols=1)
    else:
        fig, ax = plt.subplots(nrows=4, ncols=1)
    # fig.set_figwidth(30)
    # fig.set_figheight(30)

    # fig.suptitle('This is a somewhat long figure title', fontsize=16)

    if merged.attrs['res']:
        title = f'SNOTEL and S1 Snow Depth Averaged over {merged.attrs['res']} m Resolution of the {merged.attrs['location'].capitalize()} during the {st_date} to {ed_date} Period'
    else:
        title = f'SNOTEL and S1 Snow Depth Averaged over the AOI of the {merged.attrs['location'].capitalize()}) during the {st_date} to {ed_date} Period'

    fig.suptitle(title, fontsize=24)

    # draw s1 and SNOTEL snow depth on the same panel
    fcf_ratio = float("{:.2f}".format(merged.attrs['fcf_ratio']))
    ax[0].plot(merged['date'], merged['snow_depth_x'], label=f'SNOTEL Snow Depth, R = {corr}, FC ratio {fcf_ratio}, S1-SNOTEL: min {min}, mean {mean}, max {max}, std {std}', color='red')
    ax[0].plot(merged1['date'], merged1['snow_depth_y'], label=f'Averaged Snow Depths of S1, xsize={merged.attrs['xsize']}, ysize={merged.attrs['ysize']}', color='blue')
    ax[0].legend()

    # temperature and 32F line
    line32 = np.zeros_like(merged['date'], dtype=float)
    line32[:] = 32.0
    ax[1].plot(merged['date'], merged['temp_avg'], label='Temperature', color='red')
    ax[1].plot(merged['date'], line32, label='32 F', color='blue')
    ax[1].legend()

    # cross-polarization ratio
    ax[2].plot(merged1['date'], merged1['cr_mean'], label='Cross-Polarization Ratio', color='blue')
    ax[2].legend()

    # precipitation
    ax[3].plot(merged['date'], merged['precip_inc'], label='precip increment, m', color='red')
    ax[3].legend()

    # snow-rain-ratio
    if 'snow_rain_ratio' in merged.columns:
        ax[4].plot(merged['date'], merged['snow_rain_ratio'], label='snow-rain-ratio',color='blue')
        ax[4].legend()

    if outfile:
        # create directory if not exist
        Path(outfile).parent.mkdir(exist_ok=True)
        fig.savefig(outfile)

    plt.show()

def draw_multiple_bars(merged, outfile):
    # when calculating corr, get rid of NAN data
    merged = merged[~merged['snow_depth_y'].isnull()]
    corr = merged['snow_depth_y'].corr(merged['snow_depth_x'])
    corr = float("{:.2f}".format(corr))

    st_date = merged['Time_x'].iloc[0].strftime('%Y-%m-%d')
    ed_date = merged['Time_x'].iloc[-1].strftime('%Y-%m-%d')

    # set global parameters for the figure
    params = {'legend.fontsize': 20, 'legend.handlelength': 2,
              'axes.titlesize': 22,
              'axes.labelsize': 20,
              'xtick.labelsize': 20,
              'ytick.labelsize': 20
              }
    plt.rcParams.update(params)

    # if 'snow_rain_ratio' in merged.columns:
    #    fig, [ax0, ax1, ax2, ax3, ax4] = plt.subplots(nrows=5, ncols=1)
    # else:

    # fig, [ax0, ax1, ax2, ax3] = plt.subplots(nrows=4, ncols=1)

    fig, ax = plt.subplots(nrows=4, ncols=1)
    fig.set_figwidth(30)
    fig.set_figheight(30)
    # fig.suptitle('This is a somewhat long figure title', fontsize=16)

    if merged.attrs['res']:
        title = f'SNOTEL and S1 Snow Depth Averaged over {merged.attrs['res']} m Resolution of the {merged.attrs['location'].capitalize()} during the {st_date} to {ed_date} Period'
    else:
        title = f'SNOTEL and S1 Snow Depth Averaged over the AOI of the {merged.attrs['location'].capitalize()} during the {st_date} to {ed_date} Period'

    fig.suptitle(title, fontsize=24)

    # increase the size of labels
    # [i.tick_params(axis='both', which='major', labelsize=20) for i in ax]
    # [i.tick_params(axis='both', which='minor', labelsize=20) for i in ax]



    time = pd.to_datetime(merged['Time_x'])
    # time = time.dt.strftime('%d')
    x = np.arange(len(time))

    width = 0.25  # the width of the bars
    multiplier = 0

    snow_depths = {'s1_snow_depth': merged['snow_depth_y'], 'snotel_snow_depth': merged['snow_depth_x']}

    for attribute, measurement in snow_depths.items():
        offset = width * multiplier
        if multiplier == 0:
            lbl = f'{attribute}, R={corr}'
        else:
            lbl = f'{attribute}'

        rects = ax[0].bar(x + offset, measurement, width, label=lbl)
        # ax0.bar_label(rects, padding=3)
        multiplier += 1

    # Add some text for labels, title and custom x-axis tick labels, etc.
    ax[0].set_ylabel('Snow Depth, m')
    ax[0].set_title('S1 and SNOTEL Snow Depth')

    xtks = ax[0].get_xticks()

    xtks1 =xtks[1: len(xtks)-1]

    xtknames1 = [time[time.index[int(x[int(i)])]].strftime('%Y-%m-%d') for i in xtks1]

    ax[0].set_xticks(xtks1 + width, xtknames1)
    #ax[0].tick_params(axis='both', which='major', labelsize=20)
    #ax[0].tick_params(axis='both', which='minor', labelsize=15)
    ax[0].legend(loc='upper left', ncols=2)
    # ax[0].set_ylim(0, 250)

    # temperature and 32F line
    merged_line32 = np.zeros_like(x, dtype=float)
    merged_line32[:] = 32.0
    # ax1.plot(snotel['Time'], snotel['temp_avg'], label='Temperature', color='red')
    ax[1].plot(x, merged['temp_avg'], label='Daily Average Air Temperature', color='red')
    ax[1].plot(x, merged_line32, label='32 F', color='blue')
    ax[1].set_ylabel('Temperature, degree F')
    ax[1].set_xticks(xtks1, xtknames1)
    ax[1].legend()

    # cross-polarization ratio
    ax[2].plot(x, merged['cr_mean'], label='Cross-Polarization Ratio', color='blue')
    ax[2].set_xticks(xtks1, xtknames1)
    ax[2].set_ylabel('VH/VV Ratio')
    ax[2].legend()

    # precipitation
    # ax3.plot(snotel['Time'], snotel['precip_inc'], label='precip increment, m', color='red')
    # ax3.plot(x, merged['precip_inc'], label='precip increment, m', color='red')
    rects = ax[3].bar(x, merged['precip_inc'], width, label = 'Precipitation Increment')
    # ax3.bar_label(rects, padding=3)
    ax[3].set_xticks(xtks1, xtknames1)
    ax[3].set_ylabel('Precipitation, m')
    ax[3].legend()

    '''
    # snow-rain-ratio
    if 'snow_rain_ratio' in merged.columns:
        # ax4.plot(snotel['Time'], snotel['snow_rain_ratio'], label='snow-rain-ratio',color='blue')
        # ax4.plot(merged['Time'], merged['snow_rain_ratio'], label='snow-rain-ratio', color='blue')
        data={'snow_rain_ratio': merged['snow_rain_ratio']}
        rects = ax4.bar(x,  data['snow_rain_ratio'], width, label='snow_rain_ratio')
        ax4.set_ylabel('Ratio')
        ax4.set_title('Snow Rain Ratio')
        ax4.set_xticks(x + width, time)
        ax4.legend()
    '''

    if outfile:
        # create directory if not exist
        Path(outfile).parent.mkdir(exist_ok=True)
        fig.savefig(outfile)

    plt.show()


def investigate_s1_snotel(s1_file, snotel_file, lon, lat, aoi: None, res: None, stdate: None, eddate: None, outfile: None):
    # read the nc_ds
    # stdate and eddate are yyyy-mm-dd, typical stdate='2021-11-15', eddate='2022-03-15'

    nc_ds = xr.open_dataset(s1_file, decode_coords="all")

    # if stdate and eddate, only keep data from stdate to eddata, normally form 11/15 to 3/15
    if stdate and eddate:
        #st = pd.to_datetime(nc_ds.time[0].data)
        #st_date = st.strftime('%Y-11-15')
        #ed = pd.to_datetime(nc_ds.time[-1].data)
        #ed_date = ed.strftime('%Y-03-15')
        nc_ds = nc_ds.sel(time=slice(stdate, eddate))

    # test purpose
    s1_polybbox = bbox2polygon(list(nc_ds.snow_depth.rio.bounds()))
    write_polygon(s1_polybbox, s1_file.replace(".nc", "_s1_output.geojson"))

    # clip by the AOI or by box of 500m around the lon, lat
    if aoi:
        # geojsfile = "/media/jiangzhu/Elements/crrel/SNOTEL/creamers/creamers.geojson"
        poly = readgeojson(aoi)
    elif res:
        poly = polygon_via_point(lon=lon, lat=lat, resolution=res)
        write_polygon(poly, s1_file.replace(".nc", f'_rectangular_{res}.geojson'))
        # poly = geometry.box(*poly.bounds)
    else:
        poly=None

    if poly:
        jsstr = to_geojson(poly)
        geometries = [json.loads(jsstr)]
        nc_ds = nc_ds.rio.clip(geometries, all_touched=True)
        xsz = len(nc_ds.snow_depth.x)
        ysz = len(nc_ds.snow_depth.y)
        tot_num = nc_ds.fcf.data.size
        forest_num = nc_ds.fcf.data[nc_ds.fcf.data>=0.5].size
        fcf_ratio = forest_num/float(tot_num)

    # test purpose

    s1_clipped = bbox2polygon(list(nc_ds.snow_depth.rio.bounds()))
    write_polygon(s1_clipped, s1_file.replace(".nc", "_s1_clipped.geojson"))

    # calculate the cr, cr_mean
    s1_sd_mean = nc_ds.snow_depth.mean(axis=(0,1))
    vh = 10**nc_ds.s1.sel(band='VH')
    vh_mean = vh.mean(axis=(1,2))
    vv = 10**nc_ds.s1.sel(band='VV')
    vv_mean= vv.mean(axis=(1,2))
    cr = vh/vv
    cr_mean = cr.mean(axis=(1,2))

    # add cr_mean to s1_sd_mean
    s1_merged = xr.merge([s1_sd_mean, cr_mean.rename('cr_mean')])
    s1_df = s1_merged.to_dataframe()
    s1_df['Time'] = s1_df.index
    # get location
    str_list = Path(s1_file).stem.split("_")
    loc = str_list[1]
    st = str_list[2]
    ed = str_list[3]

    # Creamers' field
    # Latitude:	64.87
    # Longitude:	-147.74
    filename = str(Path(snotel_file).stem)

    if filename.find("creamers") != -1 or  filename.find("fieldinglake") != -1:
        snotel = read_snotel_sd_precip_ratio_temp(snotel_file, lon, lat, {})
    else:
        snotel = read_snotel_sd_precip_temp(snotel_file, lon, lat, {})

    # convert date string to datetime64
    snotel['Time'] = pd.to_datetime(snotel['Time'])

    # only consider stdate to eddate period
    if stdate and eddate:
        mask = (snotel['Time']>=stdate) & (snotel['Time']<=eddate)
        snotel = snotel[mask]

    snotel = snotel.set_index(['time'])

    # merge snotel1 to s1_df
    # merged = pd.merge_asof(s1_df, snotel, on="time", direction="nearest")

    snotel['date'] = snotel.index.date
    s1_df['date'] = s1_df.index.date

    merged = pd.merge(snotel, s1_df, on='date', how='outer')

    # snotel with x, s1_df with y
    # remove NaT in Time_x
    merged  = merged[~merged.Time_x.isnull()]

    # add attrs to merged
    merged.attrs['location'] = loc
    merged.attrs['lon'] = lon
    merged.attrs['lat'] = lat
    merged.attrs['st_date'] = stdate
    merged.attrs['ed_date'] = eddate
    merged.attrs['res'] = res
    merged.attrs['xsize'] = xsz
    merged.attrs['ysize'] = ysz
    merged.attrs['fcf_ratio'] = fcf_ratio
    # output the dataframe to csv
    merged.to_csv(outfile.replace('.png','.csv'))

    # calculate corr
    merged1 = merged[~merged['snow_depth_y'].isnull()]
    corr = merged1['snow_depth_y'].corr(merged1['snow_depth_x'])
    corr = float("{:.2f}".format(corr))


    # draw plots
    outfile = outfile.replace('.png', f'_x{xsz}_y{ysz}.png')

    draw_multiple_lines(merged, 'Creamers', outfile)
    # draw bar plots
    draw_multiple_bars(merged, outfile.replace('.png','_barplots.png'))



def main():

    #outfile = '/home/jiangzhu/data/crrel/SNOTEL/creamers/investigate.png'

    #s1_file = "/home/jiangzhu/data/crrel/SNOTEL/creamers/s1_creamers_20230801_20240731.nc"

    #snotel_file = "/home/jiangzhu/data/crrel/SNOTEL/creamers/depth_precipt_temp_20230801_20240731.txt"

    #lon = -147.74
    #lat = 64.87
    #attrs = {}

    parser = ArgumentParser()
    parser.add_argument('--s1file', type=str,  required=True)
    parser.add_argument('--snotelfile', type=str, required=True)
    parser.add_argument('--lon', type=float, required=True)
    parser.add_argument('--lat', type=float, required=True)
    parser.add_argument('--aoigeojsonfile', type=str, default=None)
    parser.add_argument('--res', type=float, default=None)
    parser.add_argument('--stdate', type=str, default=None)
    parser.add_argument('--eddate', type=str, default=None)
    parser.add_argument('--outfile', type=str, default=None)

    args = parser.parse_args()

    if args.aoigeojsonfile is None and args.res is None:
        print("need to input either aoigeojsonfile or res")
        sys.exit(1)
    investigate_s1_snotel(args.s1file, args.snotelfile, args.lon, args.lat, args.aoigeojsonfile, args.res, args.stdate, args.eddate, args.outfile)

    print('completed ...')

if __name__ == '__main__':
    main()








import shapely
from pathlib import Path
import numpy as np
import pandas as pd
import xarray as xr
import rioxarray as rxa
from rasterio.enums import Resampling

from datetime import datetime

from asf_analysis.s1_lidar.download_lidar import download_lidar_data

from asf_analysis.s1_lidar.make_spicy_ds import make_site_ds

from asf_analysis.s1_lidar.rmse_table import stats_table_new, stats_table_old


# 1. download the Lidar data

lidar_dir = Path('/home/jiangzhu/data/crrel/lidar2')

# lidar_dir.mkdir(parents=True, exist_ok = True)

scratch_dir = Path.joinpath(lidar_dir,"scratch")

scratch_dir.mkdir(parents=True, exist_ok=True)

# check if there are *_DEM_*, "*_VH_*, and *_SD_* files

lst1 = [f for f in lidar_dir.glob('*_DEM_*')]
lst2 = [f for f in lidar_dir.glob('*_SD_*')]
lst3 = [f for f in lidar_dir.glob('*_VH_*')]

if  len(lst1) == 0 or len(lst2) == 0 or len(lst3) == 0:
    download_lidar_data(lidar_dir)

# 2. construct the lidar ds

lidar_file = f'{lidar_dir}/lidar_ds_site.nc'

sites = {'USIDBS': 'Banner'}

site = 'USIDBS'

site_name = 'Banner'

filename_filter = '*20200218_20200219*'

if Path(lidar_file).is_file():
    lidar_ds = xr.load_dataset(lidar_file, decode_coords="all")
else:
    for site, site_name in sites.items():
        lidar_ds = make_site_ds(site, lidar_dir, filename_filter=filename_filter)
        lidar_ds.to_netcdf(lidar_file)

# cut off no-sense data

#lidar_ds = lidar_ds.where(lidar_ds < 1e9).where(lidar_ds >= 0) # not work, killed

ch = np.logical_and(lidar_ds < 1e9, lidar_ds >= 0)
lidar_ds['lidar-sd'] = lidar_ds['lidar-sd'].where(ch['lidar-sd'])
lidar_ds['lidar-vh'] = lidar_ds['lidar-vh'].where(ch['lidar-vh'])
lidar_ds['lidar-dem'] = lidar_ds['lidar-dem'].where(ch['lidar-dem'])
lidar_ds.to_netcdf(f'{lidar_dir}/lidar_ds_site_validdata.nc')

# 3. read in teh s1 data

s1file = f'{str(lidar_dir)}/spicy_ds.nc'

s1_ds = xr.load_dataset(s1file, decode_coords="all")

# 4. resample lidar ds based on S1 ds

# set nodata as np.nan
for var in lidar_ds:
    lidar_ds[var] = lidar_ds[var].rio.write_nodata(np.nan)

# resample lidar based on s1 data
lidar_ds = lidar_ds.rio.reproject_match(s1_ds, resampling = Resampling.average, nodata=np.nan)

# merge lidar to s1
# ds = ds[['lidar-sd', 'lidar-vh', 'lidar-dem', 'snow_depth', 's1', 'wet_snow']]
ds = xr.merge([s1_ds, lidar_ds], combine_attrs = 'drop_conflicts')

# add attributes to combined data
date = lidar_ds.time

ds.attrs['site'] = site_name
ds.attrs['site_abbrev'] = site
ds.attrs['lidar-flight-time'] = str(date[0].dt.strftime("%Y-%m-%d").values)
ds.attrs['processing-date'] = f'{datetime.now()}'

# output the combined data
out_nc = f'{str(scratch_dir)}/s1_lidar.nc'
try:
    ds.to_netcdf(out_nc)
except:
    print('Unable to create netcdf4 for {site_name}')

# call Zach's calculate functions

cvsfile_new = Path(f'{str(lidar_dir)}/stats_table_new.csv')

stats_table_new(scratch_dir, cvsfile_new)

cvsfile_old = Path(f'{str(lidar_dir)}/stats_table_new.csv')

stats_table_old(scratch_dir, cvsfile_old)

print('completed ...')








"""
Functions to download IMS snow-coverage images.

https://usicecenter.gov/Products/ImsHome
https://nsidc.org/data/user-resources/help-center/how-access-data-using-ftp-client-command-line-wget-or-python
"""

import sys
import os
from os.path import basename, exists, expanduser, join
from pathlib import Path
import shutil
from datetime import datetime
import urllib.request
from urllib.error import URLError
from tqdm import tqdm
from pyproj import Transformer
import shapely
import pandas as pd
import xarray as xr
import rioxarray as rxa

import logging
log = logging.getLogger(__name__)

sys.path.append(expanduser('~/Documents/spicy-snow'))

from spicy_snow.utils.download import url_download, decompress


def get_ims_day_data(year: str, doy: str, tmp_dir: str) -> xr.DataArray:
    """
    Download and decompress one days worth of IMS data.

    Args:
    year: Year of the data you want.
    doy: Calendar day of year you want in 'DDD' format. Range is 001 - 366.
    """

    # make temporary directory
    cd = os.getcwd()
    os.makedirs(tmp_dir, exist_ok = True)
    os.chdir(tmp_dir)

    # if we haven't found a file that works add one more to day and try again
    local_fp = None
    while not local_fp:
        try:
            if Path(f'ims{year}{doy}_1km_v1.3.nc.gz').is_file():
                local_fp = f'ims{year}{doy}_1km_v1.3.nc.gz'
            else:
                # download IMS data for this day and year
                local_fp, _ = urllib.request.urlretrieve(f'ftp://sidads.colorado.edu/pub/DATASETS/NOAA/G02156/netcdf/1km/{year}/ims{year}{doy}_1km_v1.3.nc.gz', f'ims{year}{doy}_1km_v1.3.nc.gz')
        except:
            doy = f'{int(doy) + 1:03}'
    # decompress .gz compression
    out_file = decompress(local_fp, local_fp.replace('.gz',''))
    # open as xarray dataArray
    ims = rxa.open_rasterio(out_file, decode_times = False)
    os.chdir(cd)

    return ims


def download_snow_cover(dataset: xr.Dataset, tmp_dir: str = './tmp', clean: bool = True,) -> xr.Dataset:
    """
    Download IMS snow-cover images.

    Args:
    dataset: Full dataset to add IMS data to
    tmp_dir: filepath to save temporary downloads to [default: './tmp']
    clean: Remove temporary directory after download?

    Returns:
    None
    """
    # get list of days that we have Sentinel-1 data for
    days = [pd.to_datetime(d) for d in dataset.time.values]

    all_ims = []
    for day in tqdm(days, desc = 'Downloading IMS snow-cover'):
        # download IMS data for this day and open with xarray
        ims = get_ims_day_data(day.year, f'{day.dayofyear:03}', tmp_dir = tmp_dir) #revert to clean = True at somepoint
        # add timestamp info
        ims = ims.assign_coords(time = [day])
        # reproject and clip to match dataset
        ims = ims.rio.reproject_match(dataset['s1'])
        # add day to list of ims days
        all_ims.append(ims)
    # make dataArray of all IMS images
    full_ims = xr.concat(all_ims, dim = 'time')

    # add dataArray of IMS to the sentinel-1 dataset with timestamp information
    dataset = xr.merge([dataset, full_ims.rename('ims')])
    
    # remove data directory of downloaded IMS tifs
    if clean == True:
        shutil.rmtree(tmp_dir)

    dataset.to_netcdf(f'{tmp_dir}/sentinel1_snow.nc')

    return dataset

# End of file


import logging
import os
import sys
from importlib.metadata import version
from pathlib import Path

# import pdb


'''
if sys.version_info >= (3, 8):
    from importlib import metadata
else:
    from importlib_metadata import metadata
'''
'''
pdb.set_trace()
'''

__version__ = version(__name__)

__all__ = [
    '__version__',
]



import shapely
from pathlib import Path
import numpy as np
import pandas as pd
import xarray as xr
import rioxarray as rxa
from rasterio.enums import Resampling

from datetime import datetime

from spicy_snow.retrieval import retrieve_snow_depth

from spicy_snow.download.snowex_lidar import download_dem, download_snow_depth,\
      download_veg_height

from spicy_snow.IO.user_dates import get_input_dates


lidar_dir = Path('/bsuhome/zacharykeskinen/scratch/spicy/lidar')

sites = {'USCOCP': 'Cameron', 'USCOFR': 'Frasier', 'USIDBS': 'Banner', 
         'USIDDC': 'Dry_Creek', 'USIDMC': 'Mores', 'USUTLC': 'Little_Cottonwood'}

def make_site_ds(site: str, lidar_dir: Path) -> xr.Dataset:
    """
    Makes a dataset of snow depth, veg height, and DEM for a specific site abbreviation
    in the lidar directory. Returns it reprojected to EPSG4326.

    Args:
    site: Site abbreviation to search for
    lidar_dir: Direction of lidar tiffs

    Returns:
    dataset: xarray dataset of dem, sd, vh for site
    """

    dataset = xr.Dataset()

    for img_type in ['SD', 'VH', 'DEM']:
        # files = glob(join(lidar_dir, f'*_{img_type}_*_{site}_*.tif'))
        files = lidar_dir.glob(f'*_{img_type}_*_{site}_*.tif')
        assert files, f"No files found for {img_type} at {site}"
        
        imgs = []
        for f in files:
            print(f)

            if img_type != 'DEM':
                date = pd.to_datetime(f.stem.split('_')[-2])
                sd_date = date
            else:
                date = sd_date

            img = rxa.open_rasterio(f)

            img = img.squeeze(dim = 'band')
            
            if img_type != 'DEM':
                img = img.expand_dims(time = [date])

            imgs.append(img)
        
        if img_type != 'DEM':
            dataset['lidar-' + img_type.lower()] = xr.concat(imgs, dim = 'time')
        else:
            dataset['lidar-' + img_type.lower()] = imgs[0]

    dataset = dataset.rio.reproject('EPSG:4326')
    
    return dataset

# param_set = Path('/bsuhome/zacharykeskinen/spicy-analysis/results/params/stat_res_0.5tree.csv')
# param_df = pd.DataFrame()
# if param_set.exists():
#     param_df = pd.read_csv(param_set, index_col = 0)

for site, site_name in sites.items():
    print(''.center(40, '-'))
    print(f'Starting {site_name}')

    lidar_ds_site = make_site_ds(site, lidar_dir = lidar_dir)

    lidar_ds_site = lidar_ds_site.where(lidar_ds_site < 1e9).where(lidar_ds_site >= 0)

    area = shapely.geometry.box(*lidar_ds_site.rio.bounds())

    for date in lidar_ds_site.time:

        # if param_df.size > 0:
        #     out_dir = Path('/bsuhome/zacharykeskinen/scratch/spicy/SnowEx-Data-params/')
        # else:
        out_dir = Path('/bsuhome/zacharykeskinen/scratch/spicy/SnowEx-Data-lave/')
        out_dir.mkdir(exist_ok = True)
        
        out_nc = out_dir.joinpath(f'{site_name}_{(date).dt.strftime("%Y-%m-%d").values}.nc')

        if out_nc.exists():
            print(f'Outfile {out_nc} exists already.')
            continue

        print(f'Starting {site_name} snow depth @ {date.values}')

        if date.dt.month > 4:
            continue

        lidar_ds = lidar_ds_site.sel(time = date)

        dates = get_input_dates(date.data + pd.Timedelta('14 day'))

        # if param_df.size > 0:
        spicy_ds = retrieve_snow_depth(area = area, dates = dates, work_dir = '/bsuhome/zacharykeskinen/scratch/spicy/', \
                                        job_name = f'spicy_{site}_{dates[1]}', existing_job_name = f'spicy_{site}_{dates[1]}', \
                                        params = [1.5, 0.1, 0.59])
        # else:    
            # spicy_ds = retrieve_snow_depth(area = area, dates = dates, work_dir = '/bsuhome/zacharykeskinen/scratch/spicy/', job_name = f'spicy_{site}_{dates[1]}', existing_job_name = f'spicy_{site}_{dates[1]}')

        for var in lidar_ds:
            lidar_ds[var] = lidar_ds[var].rio.write_nodata(np.nan)
        lidar_ds = lidar_ds.rio.reproject_match(spicy_ds, resampling = Resampling.average, nodata=np.nan)

        ds = xr.merge([spicy_ds, lidar_ds], combine_attrs = 'drop_conflicts')

        # ds = ds[['lidar-sd', 'lidar-vh', 'lidar-dem', 'snow_depth', 's1', 'wet_snow']]

        ds.attrs['site'] = site_name
        ds.attrs['site_abbrev'] = site
        ds.attrs['lidar-flight-time'] = str((date).dt.strftime("%Y-%m-%d").values)
        ds.attrs['processing-date'] = f'{datetime.now()}'
        
        try:
            ds.to_netcdf(out_nc)
        except:
            print('Unable to create netcdf4 for {site_name}')


from asf_snow.utils.compare_sentinel1_snow_depth_to_snex_lidar import *

file="/home/jiangzhu/data/crrel/SnowEx20-21_QSI_Lidar_Snow_Depth_0d5m_UTM_Grid_V1/store.pckl"

f = open(file, 'rb')

lidar, nc = pickle.load(f)

f.close()

scatter_plot(lidar,nc, '/home/jiangzhu/data/crrel/SnowEx20-21_QSI_Lidar_Snow_Depth_0d5m_UTM_Grid_V1/test_scatter.png')





from pathlib import Path
import shapely
import json
import logging
import os
import sys
from typing import Optional
from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser

from asf_snow.utils import utils
# Add main repo to path if you haven't added with conda-develop
# import sys
# sys.path.append('path/to/the/spicy-snow/')

# from spicy_snow.retrieval import retrieve_snow_depth
from asf_snow.retrieval_sentinel1_snow import retrieve_snow_depth
from spicy_snow.IO.user_dates import get_input_dates


def sentinel1_snow(area: list, dates: list, workdir=None, jobname=None, existjobname=None, outnc=None, debug=False,
                   parameterabc=[2.5, 0.2, 0.55]):
    """ Calculate the snow depths with sentinel-1 c-band data vv,vh,and incidental angle.
    :param area: list [minlon,minlat,maxlon,maxlat]
    :param date: yyyy-mm-dd
    :param workdir: path
    :param jobname: job name
    :param existjobname: existing job name
    :param outnc: path of output nc file
    :param debug: if run the program in debug.
    :param: parameterabc: [A,B,C], default [2.5, 0.2, 0.55]
    :return:
    """
    '''
    environment_jsonfile = f'{workdir}/tmp/environment.json'
    log = logging.getLogger(__name__)
    if Path(environment_jsonfile).is_file():
        # compare function arguments with the context in the environment.json
        environment = utils.read_environment(environment_jsonfile)
        if not (environment['area'] == area and environment['daterange'] == dates and environment['workdir'] == workdir):
            log.info(f'{workdir} is not empty, you need use other work directory')
            sys.exit(1)
    '''
    log = logging.getLogger(__name__)

    environment = json.loads(os.environ['ENVIRONMENT'])
    # if not (environment['area'] == area and environment['daterange'] == dates and environment['workdir'] == workdir):
    #     log.info(f'{workdir} is not empty, you need use other work directory')
    #     sys.exit(1)

    if not (environment['area'] == area and environment['daterange'] == dates):
        log.info(f'{workdir} is not empty, you need use other work directory')
        sys.exit(1)

    # change to your minimum longitude, min lat, max long, max lat
    area1 = shapely.geometry.box(area[0], area[1], area[2], area[3])

    Path(workdir).mkdir(parents=True, exist_ok=True)

    # this will be where your results are saved
    out_nc = Path(outnc).expanduser()

    # this will generate a tuple of dates from the previous August 1st to this date
    # dates = get_input_dates(dates)  # run on all s1 images from (2020-08-01, 2021-04-01) in this example
    dates = tuple(dates)
    spicy_ds = retrieve_snow_depth(area=area1, dates=dates, work_dir=Path(workdir).expanduser(),
                                   job_name=jobname,
                                   existing_job_name=existjobname,
                                   outfp=out_nc,
                                   debug=False,
                                   params=parameterabc,
                                   )
    return spicy_ds


def check_earthdata_credentials(username, password):
    if username is None:
        username = os.getenv('EARTHDATA_USERNAME')
        if username is None:
            raise ValueError(
                'Please provide Earthdata username via the --username option '
                'or the EARTHDATA_USERNAME environment variable.'
            )

    if password is None:
        password = os.getenv('EARTHDATA_PASSWORD')
        if password is None:
            raise ValueError(
                'Please provide Earthdata password via the --password option '
                'or the EARTHDATA_PASSWORD environment variable.'
            )

    return username, password


def main():
    parser = ArgumentParser()
    parser.add_argument('--username', type=str, default=None)
    parser.add_argument('--password', type=str, default=None)
    parser.add_argument('--area', nargs="*", type=float, default=[-113.2, 43, -113, 43.4])
    parser.add_argument('--daterange', nargs="*", type=str, default=['2008-08-01', '2021-04-01'])
    parser.add_argument('--workdir', type=str, default='.')
    parser.add_argument('--jobname', type=str, default='testing_job')
    parser.add_argument('--existjobname', type=str, default='testing_job')
    parser.add_argument('--outnc', default='test.nc')
    parser.add_argument('--debug', type=bool, default=False)
    parser.add_argument('--parameterabc', nargs="*", type=float, default=[2.5, 0.2, 0.55])

    args = parser.parse_args()

    # write args if args.workdir not exist

    environment_jsonfile = f'{args.workdir}/tmp/environment.json'

    if not Path(environment_jsonfile).is_file():
        environment = vars(args)
        environment['step'] = 'begin'
        os.environ['ENVIRONMENT'] = json.dumps(environment)

    else:
        environment = utils.read_environment(environment_jsonfile)
        os.environ['ENVIRONMENT'] = json.dumps(environment)

       # set environment for share
    os.environ['EARTHDATA_USERNAME'] = args.username
    os.environ['EARTHDATA_PASSWORD'] = args.password

    # username, password = check_earthdata_credentials(args.username, args.password)

    logging.basicConfig(format='%(asctime)s - %(levelname)s - %(message)s',
                        datefmt='%m/%d/%Y %I:%M:%S %p', level=logging.INFO)

    # write_credentials_to_netrc_file(username, password)

    spicy_ds = sentinel1_snow(area=args.area, dates=args.daterange, workdir=args.workdir, jobname=args.jobname,
                              existjobname=args.existjobname, outnc=args.outnc, parameterabc=args.parameterabc)


if __name__ == '__main__':
    main()





from asf_snow.utils.analyze_sentinel1_snow import *
from asf_snow.utils.analyze_snex import *
import geopandas as gpd
import numpy as np

s1file = "/home/jiangzhu/data/crrel/SNOTEL/creamersfield/s1_2/sentinel1_creamers_20220801_20230731n_10_12h_12h_s1_average_y_creamers_field.shp"

snotelfile="/home/jiangzhu/data/crrel/SNOTEL/creamersfield/20220801_20230731.csv"

lon = -147.733

lat =  64.867

snotel = read_snotel(snotelfile, lon, lat)
snotel = snotel[~snotel['snow_depth'].isnull()]

snotel.time = snotel['time'] -  np.timedelta64(9, 'h')

snotel_ts = np.array([item.timestamp() for item in snotel['time']])
#snotel_t = np.array(snotel.time)
#snotel_t1 = snotel_t - np.timedelta64(9, 'h')

s1 = gpd.read_file(s1file)
s1 = s1[~s1['mean'].isnull()]

# s1_t = np.array(s1.time)
s1_ts = np.array([item.timestamp() for item in s1['time']])

s1_mean = np.array(s1['mean'])
s1_min = np.array(s1['min'])
s1_max = np.array(s1['max'])
s1_std = np.array(s1['std'])

snotel_sd = np.array(snotel['snow_depth'])

# interpolate s1 based on times of snotel

s1_interp_mean = np.interp(snotel_ts, s1_ts, s1_mean)
s1_interp_min = np.interp(snotel_ts, s1_ts, s1_min)
s1_interp_max = np.interp(snotel_ts, s1_ts, s1_max)
s1_interp_std = np.interp(snotel_ts, s1_ts, s1_std)


num = len(snotel_ts)

geometry = np.array([s1['geometry'].iloc[0]]*num)

columns = ['time','mean', 'min','max', 'std', 'snex_sd', 'geometry']

parsed =[[snotel_ts[i], s1_interp_mean[i], s1_interp_min[i],
          s1_interp_max[i], s1_interp_std[i], snotel_sd[i], geometry[i]] for i in range(num)]

# create new geoDataframe
gdf = gpd.GeoDataFrame(data=parsed, columns=columns)

gdf = gdf.set_crs('epsg:4326')


# calculate RMSE, R, and MBE
xm = snotel_sd
ym = s1_interp_mean

r = np.corrcoef(ym, xm)
corr = r[0,1]
rmse = np.sqrt(((xm - ym) ** 2).mean())
mbe = np.mean(ym - xm)

stderror =np.array(gdf['std'])

# draw plots
fig, [ax0, ax1] = plt.subplots(nrows=2, ncols=1)

fig.set_figwidth(30)
fig.set_figheight(30)

# mean, obs
s1_mean = "{:.2f}".format(gdf['mean'].mean())
s1_std = "{:.2f}".format(gdf['mean'].std())
obs_mean = "{:.2f}".format(gdf['snex_sd'].mean())
obs_std = "{:.2f}".format(gdf['snex_sd'].std())
corr1 = float("{:.2f}".format(corr))
rmse1 = "{:.2f}".format(rmse)
mbe1 = "{:.2f}".format(mbe)

ax0.plot(gdf.index, gdf['mean'], label=f'S1 Snow Depth, mean {s1_mean},'
                                       f' std {s1_std} ', color='blue')
ax0.errorbar(gdf.index, gdf['mean'], yerr=stderror, color='blue')

ax0.plot(gdf.index, gdf['snex_sd'], label=f'SNOTEL Snow Depth, mean {obs_mean},'
                                          f'std {obs_std}', color='red')
ax0.legend()

ax0.text(0.3, 0.9, f'RMSE {rmse1}, R {corr1}, MBE {mbe1}, Number of observations {len(gdf)}', color='red', transform=ax0.transAxes)

ax0.set_title(f'Zonal Statistic of S1 Snow Depth and SNOTEL Snow Depth')
ax0.set_xlabel(f'Sequence of Observations')
ax0.set_ylabel("snow depth, m")

# differnec with mean standard error bar
diff = gdf['mean'] - gdf['snex_sd']
#diff_mean = "{:.2f}".format(diff.mean())
diff_mean = round(diff.mean(), 2)
#diff_std = "{:.2f}".format(diff.std())
diff_std = round(diff.std(), 2)
ax1.errorbar(gdf.index, diff, yerr=stderror, label='Difference with Mean Standard Error', color='red')

ax1.legend()

corr1 = float("{:.2f}".format(corr))

ax1.text(0.6, 0.9, f'Correlation {corr1}, Mean of Difference {diff_mean}, STD {diff_std}', color='red',
         transform = ax1.transAxes)

ax1.set_title(f'The Difference of S1 Snow Depth Depth and SNOTEL Snow Depth')

ax1.set_xlabel(f'Sequence of Observations')

ax1.set_ylabel("Snow Depth, m")

# Number of pixels of mean
#ax2.plot(gdf.index, gdf['count'], label='Number of pixels', color='green')
# ax2.set_xticks(xtks, labels=xtkslb)
#ax2.set_xlabel(f'Sequence of Observations')
#ax2.legend()

outfile = s1file.replace(".shp", "_interp.png")
# create directory if not exist
Path(outfile).parent.mkdir(exist_ok=True)

fig.savefig(outfile)

plt.show()






print("completed ...")



from shapely import geometry
from itertools import product
for lon_min, lat_min in product(range(-117, -113), range(43, 46)):
    area = shapely.geometry.box(lon_min, lat_min, lon_min + 1, lat_min + 1)
    out_nc = Path(f'~/Desktop/spicy-test/swath_{lon_min}-{lon_min + 1}_{lat_min}-{lat_min + 1}.nc').expanduser()
    if out_nc.exists():
        continue

    spicy_ds = retrieve_snow_depth(area = area, dates = dates, 
                                work_dir = Path('~/scratch/spicy-lowman-quadrant/data/').expanduser(), 
                                job_name = f'spicy-lowman-{lon_min}-{lon_min + 1}_{lat_min}-{lat_min + 1}', # v1
                                existing_job_name = f'spicy-lowman-{lon_min}-{lon_min + 1}_{lat_min}-{lat_min + 1}', # v1
                                debug=False,
                                outfp=out_nc)




from folium import GeoJson
from geopandas import GeoSeries
from shapely import geometry, to_geojson
from zipfile import ZipFile
import json
from pathlib import Path
from zipfile import ZipFile
import geopandas as gpd
import xarray as xr
import rioxarray as rio


def bbox2polygon(bbox):
    # bbox = [ -116.12351, 37.9071, -105.86093, 47.0607]
    return geometry.box(*bbox)


def extractfromzipfile(zipfile: Path, file: Path):
    # loading the temp.zip and creating a zip object
    with ZipFile(zipfile, 'r') as zObject:
        # Extracting specific file in the zip
        # into a specific location.
        extfile = f'{zipfile.stem}/{file.name}'
        zipinfo = zObject.getinfo(extfile)
        zipinfo.filename = Path(extfile).name
        zObject.extract(zipinfo, path=file.parent)
    if file.is_file():
        return file
    else:
        return None


def write_environment(dic: dict, jsonfile):
    with open(jsonfile, 'w') as convert_file:
     convert_file.write(json.dumps(dic))


def read_environment(jsonfile: str):
    with open(jsonfile) as json_file:
        dic = json.load(json_file)
    return dic


def write_polygon(poly: geometry.Polygon, file: str):
    poly_gdf = gpd.GeoDataFrame(index=[0], crs='epsg:4326', geometry=[poly])
    poly_gdf.to_file(file, driver="GeoJSON")

def read_polygon(geojsonfile):
    # geojson file only include one raw, its geometry is a Polygon

    gdf = gpd.read_file(geojsonfile)

    return gdf.loc[0, 'geometry']

# write_polygon2 does not work well
def write_polygon2(poly:geometry.Polygon,file: str):
    poly_dic = to_geojson(poly)
    geojson_data = json.loads(poly_dic)
    f = open(file, 'w')
    json.dump(geojson_data, f)
    f.close()

def poly_coords_2_polygon(coords):

    # coord =[(-147.739642245038, 64.864684437575), (-147.739207102818, 64.8648316353217), (-147.738892557103, 64.8656067041273), (-147.739110131994, 64.8658115743219), (-147.742841408334, 64.8664873680826), (-147.745829871823, 64.8669489866227), (-147.749026104411, 64.8670220774216), (-147.750556203041, 64.8669562783539), (-147.75041963082, 64.8657147933085), (-147.74839248122, 64.8652114488119), (-147.746463756512, 64.864688434635), (-147.744731279647, 64.8644809492782), (-147.74305235255, 64.8644505200646), (-147.739642245038, 64.864684437575)]

    polygon = Polygon(coords)

    return polygon


def geometry2geojsonfile(geometry: geometry, geojsonfile: str):
    with open(geojsonfile, 'w') as f:
        js_str = to_geojson(geometry)
        js = json.loads(js_str)
        f.write(json.dumps(js))


def geometry2geojsonfile_via_geopandas(geometry: geometry, epsg_code, geojsonfile: str):
    gdf = gpd.GeoDataFrame(index=[0], crs=f'epsg:{epsg_code}', geometry=[geometry])
    gdf.set_crs(f'epsg:{epsg_code}')
    gdf.to_file(geojsonfile, driver=GeoJson)


def write_raster(data: xr.DataArray, file):
    data.rio.to_raster(file)



def boundingbox_from_gt(geoTransform: list, xsize, ysize):
    """
    given geotransform list and size list, return bounding box
    :param geoTransform:
    :param xsize
    :param ysize
    :return: bounds [minlon, minlat, maxlon, maxlat]
    """
    minx = geoTransform[0]
    maxy = geoTransform[3]
    maxx = minx + geoTransform[1] * xsize
    miny = maxy + geoTransform[5] * ysize

    return [minx, miny, maxx, maxy]


def extract_outline_from_raster(rasterfile, outputfile):
    import rasterio as rio
    from rasterio.features import dataset_features
    import geopandas as gpd

    '''
    with rio.open(rasterfile) as ds:
        shapes = list(dataset_features(ds, bidx=1, as_mask=True, geographic=False, band=False))

        for shape in shapes:
            print(shape['geometry'])
    '''
    # Or as a GeoDataFrame
    with rio.open(rasterfile) as ds:
        gdf = gpd.GeoDataFrame.from_features(dataset_features(ds, bidx=1, as_mask=True, geographic=False, band=False))
        gdf.set_crs(crs=ds.crs, inplace=True, allow_override=True)
        gdf.to_file(outputfile)

    return outputfile

def shrink_geometry(shpfile, outputfile, buff= -50.0):
    import fiona
    from shapely.geometry import (shape, MultiPolygon, Polygon)
    from shapely import (to_geojson, from_geojson)
    import geopandas as gpd
    from pathlib import Path

    # c1 = fiona.open(shpfile)

    gdf_in = gpd.read_file(shpfile)

    poly_list = [item.buffer(distance=buff) for item in gdf_in['geometry']]
    if len(poly_list) == 1:
        geom = poly_list[0]
    else:
        geom = MultiPolygon(poly_list)
    #geometry = GeoSeries(poly_list)

    gdf = gpd.GeoDataFrame(index=[0], crs=gdf_in.crs, geometry = [geom])

    gdf.to_file(outputfile, driver='ESRI Shapefile')

    return poly_list




#!/usr/bin/env python3
"""
:This file:

    `vtk2gslib.py`

:Purpose:

    Convert a VTK structured grid into a GSLIB file.

:Usage:

    From a terminal, you can run the command:

        python3 vtk2gslib.py <file.vtk> <file.gslib>

    with the two arguments:
   
        <file.vtk>
            Name of the input VTK file. (Provide the complete path if
            the file is not in the directory where you are running the
            script).
        <file.gslib>
            Name of the output GSLIB file.

:Version:

    0.1 , 2015-05-12 :

        * First version

:Authors:

    Alessandro Comunian

.. notes::

    This script requires Python3.X and the numerical libraries "numpy"
    and "pandas" installed. 

.. limitations::

    See the details in the corresponding functions
"""

import numpy as np
import pandas as pd

#
# Functions
#

def vtk2numpy(file_name, dtype=None, verbose=False):
    '''
    Read a VTK ``STRUCTURED_GRID`` file with one scalar and return it
    into a numpy array.
    
    Parameters:
        in_file: string
            The name of the VTK structured grid input file.
        dtype: numpy.dtype.name, optional
            A type which should be used to "cast" the readed data and
            and the output type of the numpy array. If the default
            value is not specified, then the value extracted from the
            VTK file is used.
        verbose: bool, optional
            A flag to decide to print read information or not. The
            default is *False*. If *True*, then some verbose info is
            printed out.

    Returns:
        This function returns the tuple (data, grid), where:

        data: numpy array
            Numpy array containing the scalars read from the VTK.
        grid : dictionary
            This is a python dictionary containing information about 
            the grid (size, spacing, origin, number of points...)
            
    .. note::
        * Only structured grid files with only one scalar can be
          handled.
        * The type of the data which is recognized in the VTK format
          for the moment is only "float" or "int".

    .. warning::
        * `POINT_DATA` and `CELL_DATA` are both accepted, but you
          should take care yourself about the kind of data you are
          dealing with.
        * The VTK file should be formatted with all the data in one
          column, otherwise the pandas reader will not be able to read
          it correctly.
    '''
    try:
        with open(file_name, 'rb') as in_file:
            # A dictionary to contain info about the grid
            grid = {}
            if verbose :
                print(('\n    Reading file: "{0}"'.format(file_name)))
            # Skip the first 3 lines
            line = in_file.readline()
            line = in_file.readline()
            line = in_file.readline()
            # This line should contain the description of the DATASET
            line = in_file.readline()
            line_split = line.split()
            if line_split[1] != b"STRUCTURED_POINTS":
                print("    Error: 'vtk2numpy' can read only ", end="")
                print("    `STRUCTURED_POINTS` datasets.")
            # The order of the following keywords can be generic...
            for i in range(4):
                line = in_file.readline()
                line_split = line.split()
                if line_split[0] == b'DIMENSIONS':
                    grid['nx']= int(line_split[1])
                    grid['ny']= int(line_split[2])
                    grid['nz']= int(line_split[3])
                elif line_split[0] == b'ORIGIN':
                    grid['ox'] = float(line_split[1])
                    grid['oy'] = float(line_split[2])
                    grid['oz'] = float(line_split[3])
                elif line_split[0] == b'SPACING':
                    grid['dx'] = float(line_split[1])
                    grid['dy'] = float(line_split[2])
                    grid['dz'] = float(line_split[3])
                elif line_split[0] == b'POINT_DATA':
                    grid['type'] = 'points'
                    grid['points']  = int(line_split[1])
                elif line_split[0] == b'CELL_DATA':
                    grid['type'] = 'cells'
                    grid['cells'] = int(line_split[1])
                else:
                    print("    Error in 'vtk2numpy'. Check you keywords.")

            # Read the data type
            line = in_file.readline()
            line_split = line.split()
            data_type = line_split[2]
            if not dtype:
                # If dtype is provided as agrument, it overwrites the
                # dtype of the dataset.
                if data_type in (b'int',):
                    dtype = np.int32
                elif data_type  in (b'float',):
                    dtype = np.float64
                else:
                    dtype = None

            # Skip the line containing the information about the
            # LOOKUP_TABLE
            line = in_file.readline()

            # Read the data as a CSV file
            data_tmp = pd.read_csv(in_file, dtype=dtype,
                                   engine='c',header=None).values
            data = np.reshape(data_tmp, (grid['nx'], grid['ny'], grid['nz']),
                              order='F')

        return data, grid

    except IOError:
        print(('    Error reading file "{0}"'.format(file_name)))
        print('    Check if the file exists...')


def numpy2gslib(data, file_name, grid=None, varname="data"):
    '''
    Convert a numpy array into a GSLIB ASCII file.

    Parameters:
        data : numpy array
            The numpy array to be saved as GSLIB file.
        file_name : string
            The name of the GSLIB file where to save the data.
        grid : python dictionary, optional
            If available, should provide the information about grid
            grid origin ('ox', 'oy' and 'oz') and the spacing ('dx',
            'dy' and 'dz').  Otherwise default values are used
        varname : string, optional
            The name of the variable to be stored in the GSLIB file.

    .. note::
        * Works only with 1D, 2D or 3D numpy arrays.
        * Only one variable per file.
    '''

    try:
        nx, ny, nz = data.shape
    except ValueError:
        print('    Warning (numpy2gslib): input data considered as 2D.')
        nx, ny = data.shape
        nz = 1

    with open(file_name, 'wb') as file_obj:
        # Write the 1st line, which can be a comment, or contain some
        # useful information about the grid.
        if grid is None:
            file_obj.write(('%d %d %d 1.0 1.0 1.0 0.0 0.0 0.0\n1\n'
                            % (nx, ny, nz)
                            ).encode('utf-8'))
        else:
            file_obj.write(('%d %d %d %f %f %f %f %f %f 1\n1\n'% 
                            (nx, ny, nz,
                             grid['dx'], grid['dy'], grid['dz'], 
                             grid['ox'], grid['oy'], grid['oz'])
                            ).encode('utf-8'))

        # Write the name of the variable
        file_obj.write(('%s\n'%(varname)).encode('utf-8'))

        # Write the data
        if data.dtype in ['float']:
            np.savetxt(file_obj, np.reshape(data, nx*ny*nz, order='F'),
                       fmt='%12.4e')
        elif data.dtype in ['int', 'int32']:
            np.savetxt(file_obj, np.reshape(data, nx*ny*nz, order='F'),
                       fmt='%d')
        else:
            print("    ERROR: Wrong input dtype (only 'int' and 'float'"
                  " are OK)")
            print("           Value provided: ", data.dtype)
 

#
# Calling the functions...
#
if __name__ == '__main__':

    import sys
    import os

    # This is the name of the input VTK
    file_vtk = sys.argv[1]

    # Name of the output GSLIB file
    file_gslib = sys.argv[2]
    
    # Read the input VTK file
    print('    Reading file "{0}"'.format(file_vtk))
    data, grid = vtk2numpy(file_vtk)

    print('    Writing file "{0}"'.format(file_gslib))
    # Save the file as GSLIB
    numpy2gslib(data, file_gslib, grid)






from pathlib import Path
import sys
import glob
from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser

from asf_snow.utils.analyze_sentinel1_snow import *

from asf_snow.utils.analyze_snex import *

# compare the snow depth derived with sentinel1 to NASA SnowEx experiment
# example arguments
# ncfile = "/home/jiangzhu/data/crrel/sentinel1_snowEx21_ts_snow_pits_v1/sentinel1.nc"
# snexfile = "/home/jiangzhu/data/crrel/snowEx21_ts_snow_pits_v1/302507681/SNEX21_TS_SP_Summary_SWE_v01.csv"
# snexenvfile = "/home/jiangzhu/data/crrel/snowEx21_ts_snow_pits_v1/SNEX21_TS_SP_Summary_Environment_v01.csv"
# varname = "snow_depth"
# res = 1000  # zonal analysis square size


def compare_sentinel1_snow_depth_to_snex(ncfile, varname, snexfile, location, res, timeint, timethresh, base, method,
                                         precip):

    # sentinel1 snow depth data
    nc_var = get_var(ncfile, varname)

    # NASA snowEx summary swe data

    if '_ts_' in Path(snexfile).stem:
        gdf_snex = read_snex21_ts_summary_swe_v01(snexfile)
        # NASA snowEx summary environment data
        # snexenvfile = "/home/jiangzhu/data/crrel/snowEx21_ts_snow_pits_v1/SNEX21_TS_SP_Summary_Environment_v01.csv"

        snexenvfile = snexfile.replace("SWE", "Environment")
        if Path(snexenvfile).is_file():
            gdf_env = read_snex21_ts_summary_environment_v01(snexenvfile)
            # combine two GeoDataFrames
            gdf_snex = pd.merge(gdf_snex, gdf_env, on=['Location', 'Site', 'PitID', 'Time', 'UTMZone', 'Easting',
                                                   'Northing', 'Latitude', 'Longitude'], how='inner')

            gdf_snex = gdf_snex.rename(columns={"geometry_x": "geometry"})

            gdf_snex = gdf_snex.drop(columns=['geometry_y'])

        # get rid of rows that have precipitation of moderate or heavy
        # NONE=no snow anywhere in plot area; VERY LIGHT=occasional snowflake up to ~ 0.5 cm per hour accumulation;
        # LIGHT=~ 1 cm per hour accumulation; MODERATE=~ 2 cm per hour accumulation; HEAVY=~ 5 cm per hour accumulation
        if precip == 'n' and 'Recip Rate' in gdf_snex:
            # gdf_snex = gdf_snex[~((gdf_snex['Precip Rate'].str.lower() == 'moderate') |
            # (gdf_snex['Precip Rate'].str.lower() == 'heavy'))]
            gdf_snex = gdf_snex[gdf_snex['Precip Rate'].isnull()]

    elif '_SWE_' in Path(snexfile).stem:
        gdf_snex = read_snex23_swe(snexfile)

    elif '_comunity_' in Path(snexfile).stem:
        gdf_snex = read_SNEX23_MAR23_SD_AK_20230307_20230316_v01d0(snexfile)
    else:
        print(f'{snexfile} not exist')
        sys.exit(1)
    # go over each location
    locations = gdf_snex['Location'].unique()

    for loc in locations:
        # only check Boise River Basin
        # if loc not in 'Boise River Basin':
        #    continue
        # only check Fraser Experimental Forest
        if not (location in loc):
            continue

        gdf_snex = gdf_snex[gdf_snex['Location'].str.contains(loc)]

        gdf_snex_v = gdf_snex.sort_values(by=['time'], ignore_index=True)

        if base == 'snex':
            # call zonal_analyses_based_snex
            combined = zonal_analyses_based_snex(nc_var, gdf_snex_v, res=res, timeint=timeint, timethresh=timethresh,
                                                 method=method)
        else:
            # call zonal_analyses_based_sentinel1
            combined = zonal_analyses_based_sentinel1(nc_var, gdf_snex_v, res=res, timeint=timeint,
                                                      timethresh=timethresh, method=method)

        affix = f'_{int(res)}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_{loc.replace(" ", "_")}.shp'.lower()

        outfile = ncfile.replace(".nc", affix)

        pwildcard = str(Path(outfile).parent.joinpath(Path(outfile).stem)) + ".*"

        for file in glob.glob(pwildcard):
            # remove the file if it exists
            Path(file).unlink(missing_ok=True)

        # if not combined.empty:
        if type(combined) is not None:
            if not combined.empty:
                # save the combined GeoDataFrame
                combined.to_file(outfile)
            # break
            # draw line plot
            # pngfile = outfile.replace('.shp', '.png')
            # draw_combined_gdf(combined, res=res, timeint=timeint, timethresh=timethresh, flg=method,
            #                  outfile=pngfile, precip=precip)

    return outfile


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--ncfile', type=str,  required=True)
    parser.add_argument('--varname', type=str, default='snow_depth')
    parser.add_argument('--snexfile', type=str,  required=True)
    parser.add_argument('--location', type=str, default='Boise')
    parser.add_argument('--res', type=float, default=100)
    parser.add_argument('--timeint', type=int, default=24)
    parser.add_argument('--timethresh', type=int, default=6)
    parser.add_argument('--base', choices=['snex', 's1'], default='s1')
    parser.add_argument('--method', choices=['nearest', 'average', 'close',
                                             'nearest', 'average', 'close'])
    parser.add_argument('--precip', choices=['y', 'n'], default='y')

    args = parser.parse_args()

    compare_sentinel1_snow_depth_to_snex(args.ncfile, varname=args.varname, snexfile=args.snexfile,
                                         location=args.location, res=args.res, timeint=args.timeint,
                                         timethresh=args.timethresh, base=args.base, method=args.method,
                                         precip=args.precip)

    print('completed analysis...')



import shapely
from pathlib import Path
import pandas as pd
import xarray as xr
import rioxarray as rxa
from datetime import datetime

from spicy_snow.retrieval import retrieve_snow_depth

from spicy_snow.download.snowex_lidar import download_dem, download_snow_depth,\
      download_veg_height

from spicy_snow.IO.user_dates import get_input_dates


lidar_dir = Path('/bsuhome/zacharykeskinen/scratch/spicy/lidar')

sites = {'USIDBS': 'Banner'}

def make_site_ds(site: str, lidar_dir: Path) -> xr.Dataset:
    """
    Makes a dataset of snow depth, veg height, and DEM for a specific site abbreviation
    in the lidar directory. Returns it reprojected to EPSG4326.

    Args:
    site: Site abbreviation to search for
    lidar_dir: Direction of lidar tiffs

    Returns:
    dataset: xarray dataset of dem, sd, vh for site
    """

    dataset = xr.Dataset()

    for img_type in ['SD', 'VH', 'DEM']:
        # files = glob(join(lidar_dir, f'*_{img_type}_*_{site}_*.tif'))
        files = lidar_dir.glob(f'*_{img_type}_*_{site}_*.tif')
        assert files, f"No files found for {img_type} at {site}"
        
        imgs = []
        for f in files:
            print(f)

            if img_type != 'DEM':
                date = pd.to_datetime(f.stem.split('_')[-2])
                sd_date = date
            else:
                date = sd_date

            img = rxa.open_rasterio(f)

            img = img.squeeze(dim = 'band')
            
            if img_type != 'DEM':
                img = img.expand_dims(time = [date])

            imgs.append(img)
        
        if img_type != 'DEM':
            dataset['lidar-' + img_type.lower()] = xr.concat(imgs, dim = 'time')
        else:
            dataset['lidar-' + img_type.lower()] = imgs[0]

    dataset = dataset.rio.reproject('EPSG:4326')
    
    return dataset

for site, site_name in sites.items():
    print(''.center(40, '-'))
    print(f'Starting {site_name}')

    lidar_ds_site = make_site_ds(site, lidar_dir = lidar_dir)

    # lidar_ds_site = lidar_ds_site.where(lidar_ds_site < 1e9).where(lidar_ds_site >= 0)

    area = shapely.geometry.box(*lidar_ds_site.rio.bounds())

    for date in lidar_ds_site.time:
        if '2021' not in str(date.values.ravel()[0]):
            continue

        out_dir = Path('/bsuhome/zacharykeskinen/scratch/spicy/wet_banner/')
    
        
        out_nc = out_dir.joinpath(f'{site_name}_{(date).dt.strftime("%Y-%m-%d").values}.nc')

        out_dir.mkdir(exist_ok = True)
        if out_nc.exists():
            print(f'Outfile {out_nc} exists already.')
            continue

        print(f'Starting {site_name} snow depth @ {date.values}')

        if date.dt.month > 4:
            continue

        lidar_ds = lidar_ds_site.sel(time = date)

        dates = get_input_dates('2021-07-31', '2020-08-01')

        spicy_ds = retrieve_snow_depth(area = area, dates = dates, work_dir = '/bsuhome/zacharykeskinen/scratch/spicy/', \
                                           job_name = f'spicy_{site}_{dates[1]}', existing_job_name = f'spicy_{site}_{dates[1]}', \
                                            params = [1.1, 2.0, 0.39], freezing_snow_thresh = 1)

        lidar_ds = lidar_ds.rio.reproject_match(spicy_ds)

        ds = xr.merge([spicy_ds, lidar_ds], combine_attrs = 'drop_conflicts')

        # ds = ds[['lidar-sd', 'lidar-vh', 'lidar-dem', 'snow_depth', 's1', 'wet_snow']]

        ds.attrs['site'] = site_name
        ds.attrs['site_abbrev'] = site
        ds.attrs['lidar-flight-time'] = str((date).dt.strftime("%Y-%m-%d").values)
        ds.attrs['processing-date'] = f'{datetime.now()}'
        
        try:
            ds.to_netcdf(out_nc)
        except:
            print('Unable to create netcdf4 for {site_name}')


import numpy as np
import pandas as pd
import xarray as xr

from pathlib import Path

from itertools import product
from tqdm.contrib.itertools import product

import warnings

from scipy.stats import pearsonr
from sklearn.metrics import mean_squared_error, mean_absolute_error

def get_bias(x, y): return np.mean(x - y)

def get_stats(x, y):
    if type(x) == xr.DataArray: x = x.values.ravel()
    if type(y) == xr.DataArray: y = y.values.ravel()
    idx = (~np.isnan(x)) & (~np.isnan(y))
    x, y = x[idx], y[idx]
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", message="An input array is constant")
        r, p = pearsonr(x, y)
    b = get_bias(x, y)
    mae = mean_absolute_error(x, y)
    rmse = mean_squared_error(x, y, squared = False)
    return r, b, mae, rmse

npy_dirs = Path('/bsuhome/zacharykeskinen/scratch/spicy/param_npys')

sds = [np.load(Path(fp).joinpath('lidar.npy')) for fp in npy_dirs.glob('*_*-*-*')]
sds = np.concatenate(sds)

# Create parameter space
A = np.round(np.arange(1, 3.1, 0.1), 2)
B = np.round(np.arange(0, 2.01, 0.1), 2)
C = np.round(np.arange(0, 1.001, 0.01), 2)
locs = [fp.stem for fp in npy_dirs.glob('*_*-*-*') if (fp.is_dir())]

da = xr.DataArray(np.zeros((len(A), len(B), len(C))) , coords = [A, B, C], dims = ['A', 'B', 'C'], name = 'pearsonr')
res = xr.merge([da, da.copy().rename('mae'), da.copy().rename('rmse'), da.copy().rename('bias')])

loc_dirs = list(npy_dirs.glob('*_*-*-*'))
# for a, b, c in product(A, B, C):
#     spicy = [np.load(fp.joinpath(f'{a}_{b}_{c}.npy')) for fp in loc_dirs]
#     spicy = np.concatenate(spicy)
#     r, bias, mae, rmse = get_stats(sds, spicy)
#     for name, var in zip(['pearsonr', 'mae', 'rmse', 'bias'], [r, mae, rmse, bias]):
#         res[name].loc[dict(A = a, B = b, C = c)] = var

# res.to_netcdf(npy_dirs.joinpath('grouped.nc'))

# dry only
sds = []
for fp in loc_dirs:
    wet = np.load(fp.joinpath('wet.npy'))
    sds.append(np.load(Path(fp).joinpath('lidar.npy'))[wet == 0])
sds = np.concatenate(sds)


for a, b, c in product(A, B, C):
    spicy = []
    for fp in loc_dirs:
        wet = np.load(fp.joinpath('wet.npy'))
        spicy.append(np.load(fp.joinpath(f'{a}_{b}_{c}.npy'))[wet == 0])
    spicy = np.concatenate(spicy)
    r, bias, mae, rmse = get_stats(sds, spicy)
    for name, var in zip(['pearsonr', 'mae', 'rmse', 'bias'], [r, mae, rmse, bias]):
        res[name].loc[dict(A = a, B = b, C = c)] = var

res.to_netcdf(npy_dirs.joinpath('dry_grouped.nc'))


"""
Functions to capture, check, and return dates.
"""

from typing import Union
import numpy as np
import pandas as pd
from datetime import datetime

def get_input_dates(date2: Union[str, pd.Timestamp, np.datetime64], 
                    date1: Union[None, str, pd.Timestamp, pd.Timedelta, np.datetime64] = None):
    """
    This helper function parses user inputs and returns two dates for use in retrieval.py

    Args:
    date2: The second date to end retrievals at.
    date1: The first date to start retrievals at. If missing will be assumed to
    be last August 1st before date2

    Returns:
    retrieval_dates: Tuple of two dates in YYYY-MM-DD format to pass to retrieval
    function.
    """

    # try and parse date inputs

    # if date2 is not none then we convert to pd datetime 
    if date2:

        date2 = pd.to_datetime(date2)

        assert date2.year > 2014, f"Please enter a date after 2016 for s1 coverage.\
            Current parsed date is {date2}"
        
        assert date2 < datetime.now(), f"Please enter a date in the past"
    
    # if date1 is not none then convert to pd datetime.

    if date1:

        date1 = pd.to_datetime(date1)

        assert date1 < date2, f"Provided date 1 is later than date2."

    else:

        if date2.month < 8:
            date1 = pd.to_datetime(f'{int(date2.year - 1)}-08-01')
        else:
            date1 = pd.to_datetime(f'{int(date2.year)}-08-01')

    return date1.strftime('%Y-%m-%d'), date2.strftime('%Y-%m-%d')




"""
Utility functions for downloading files
"""
import sys
import time
from os.path import basename, exists
import gzip
from urllib.request import urlretrieve

import logging
log = logging.getLogger(__name__)

def reporthook(count, block_size, total_size):
    """
    Hook for urlib downloads to get progress readout.
    """
    global start_time
    if count == 0:
        start_time = time.time()
        return
    duration = time.time() - start_time
    progress_size = int(count * block_size)
    speed = min(int(count*block_size*100/total_size),100)
    percent = int(count * block_size * 100 / total_size)
    sys.stdout.write("\r...%d%%, %d MB, %d KB/s, %d seconds passed" %
                    (percent, progress_size / (1024 * 1024), speed, duration))
    sys.stdout.flush()

def url_download(url, out_fp, overwrite = False, verbose = True):
    """
    Downloads url with a progress bar and overwrite check.
    """
    # check if file already exists
    if not exists(out_fp) or overwrite == True:
        # progress bar for your download?
        if verbose:
            log.info(f'Downloading {basename(out_fp)}.')
            urlretrieve(url, out_fp, reporthook)
            log.info('')
        # or not?
        else:
            urlretrieve(url, out_fp)
    # if already exists. skip download.
    else:
        if verbose:
            log.info(f'{basename(out_fp)} already exists. Skipping.')

def decompress(infile, tofile):
    """
    Decompress gzipped infile to outfile location.
    """

    log.debug(f"decompressing {infile} to {tofile}")

    with open(infile, 'rb') as inf, open(tofile, 'wb') as ouf:
        decom_str = gzip.decompress(inf.read())
        ouf.write(decom_str)

        return tofile


"""
Helper functions from the nsidc
"""
from __future__ import print_function

import base64
import getopt
import itertools
import json
import math
import netrc
import os.path
import ssl
import sys
import time
from getpass import getpass

try:
    from urllib.parse import urlparse
    from urllib.request import urlopen, Request, build_opener, HTTPCookieProcessor
    from urllib.error import HTTPError, URLError
except ImportError:
    from urlparse import urlparse
    from urllib2 import urlopen, Request, HTTPError, URLError, build_opener, HTTPCookieProcessor

CMR_URL = 'https://cmr.earthdata.nasa.gov'
URS_URL = 'https://urs.earthdata.nasa.gov'
CMR_PAGE_SIZE = 2000
CMR_FILE_URL = ('{0}/search/granules.json?provider=NSIDC_ECS'
                '&sort_key[]=start_date&sort_key[]=producer_granule_id'
                '&scroll=true&page_size={1}'.format(CMR_URL, CMR_PAGE_SIZE))


def get_username():
    username = ''

    # For Python 2/3 compatibility:
    try:
        do_input = raw_input  # noqa
    except NameError:
        do_input = input

    username = do_input('Earthdata username (or press Return to use a bearer token): ')
    return username


def get_password():
    password = ''
    while not password:
        password = getpass('password: ')
    return password


def get_token():
    token = ''
    while not token:
        token = getpass('bearer token: ')
    return token


def get_login_credentials():
    """Get user credentials from .netrc or prompt for input."""
    credentials = None
    token = None

    try:
        info = netrc.netrc()
        username, account, password = info.authenticators(urlparse(URS_URL).hostname)
        if username == 'token':
            token = password
        else:
            credentials = '{0}:{1}'.format(username, password)
            credentials = base64.b64encode(credentials.encode('ascii')).decode('ascii')
    except Exception:
        username = None
        password = None

    if not username:
        username = get_username()
        if len(username):
            password = get_password()
            credentials = '{0}:{1}'.format(username, password)
            credentials = base64.b64encode(credentials.encode('ascii')).decode('ascii')
        else:
            token = get_token()

    return credentials, token


def build_version_query_params(version):
    desired_pad_length = 3
    if len(version) > desired_pad_length:
        print('Version string too long: "{0}"'.format(version))
        quit()

    version = str(int(version))  # Strip off any leading zeros
    query_params = ''

    while len(version) <= desired_pad_length:
        padded_version = version.zfill(desired_pad_length)
        query_params += '&version={0}'.format(padded_version)
        desired_pad_length -= 1
    return query_params


def filter_add_wildcards(filter):
    if not filter.startswith('*'):
        filter = '*' + filter
    if not filter.endswith('*'):
        filter = filter + '*'
    return filter


def build_filename_filter(filename_filter):
    filters = filename_filter.split(',')
    result = '&options[producer_granule_id][pattern]=true'
    for filter in filters:
        result += '&producer_granule_id[]=' + filter_add_wildcards(filter)
    return result


def build_cmr_query_url(short_name, version, time_start, time_end,
                        bounding_box=None, polygon=None,
                        filename_filter=None):
    params = '&short_name={0}'.format(short_name)
    params += build_version_query_params(version)
    params += '&temporal[]={0},{1}'.format(time_start, time_end)
    if polygon:
        params += '&polygon={0}'.format(polygon)
    elif bounding_box:
        params += '&bounding_box={0}'.format(bounding_box)
    if filename_filter:
        params += build_filename_filter(filename_filter)

    return CMR_FILE_URL + params


def get_speed(time_elapsed, chunk_size):
    if time_elapsed <= 0:
        return ''
    speed = chunk_size / time_elapsed
    if speed <= 0:
        speed = 1
    size_name = ('', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
    i = int(math.floor(math.log(speed, 1000)))
    p = math.pow(1000, i)
    return '{0:.1f}{1}B/s'.format(speed / p, size_name[i])


def output_progress(count, total, status='', bar_len=60):
    if total <= 0:
        return
    fraction = min(max(count / float(total), 0), 1)
    filled_len = int(round(bar_len * fraction))
    percents = int(round(100.0 * fraction))
    bar = '=' * filled_len + ' ' * (bar_len - filled_len)
    fmt = '  [{0}] {1:3d}%  {2}   '.format(bar, percents, status)
    print('\b' * (len(fmt) + 4), end='')  # clears the line
    sys.stdout.write(fmt)
    sys.stdout.flush()


def cmr_read_in_chunks(file_object, chunk_size=1024 * 1024):
    """Read a file in chunks using a generator. Default chunk size: 1Mb."""
    while True:
        data = file_object.read(chunk_size)
        if not data:
            break
        yield data


def get_login_response(url, credentials, token):
    opener = build_opener(HTTPCookieProcessor())

    req = Request(url)
    if token:
        req.add_header('Authorization', 'Bearer {0}'.format(token))
    elif credentials:
        try:
            response = opener.open(req)
            # We have a redirect URL - try again with authorization.
            url = response.url
        except HTTPError:
            # No redirect - just try again with authorization.
            pass
        except Exception as e:
            print('Error{0}: {1}'.format(type(e), str(e)))
            sys.exit(1)

        req = Request(url)
        req.add_header('Authorization', 'Basic {0}'.format(credentials))

    try:
        response = opener.open(req)
    except HTTPError as e:
        err = 'HTTP error {0}, {1}'.format(e.code, e.reason)
        if 'Unauthorized' in e.reason:
            if token:
                err += ': Check your bearer token'
            else:
                err += ': Check your username and password'
        print(err)
        sys.exit(1)
    except Exception as e:
        print('Error{0}: {1}'.format(type(e), str(e)))
        sys.exit(1)

    return response


def cmr_download(urls, force=False, quiet=False):
    """Download files from list of urls."""
    if not urls:
        return

    url_count = len(urls)
    if not quiet:
        print('Downloading {0} files...'.format(url_count))
    credentials = None
    token = None

    for index, url in enumerate(urls, start=1):
        if not credentials and not token:
            p = urlparse(url)
            if p.scheme == 'https':
                credentials, token = get_login_credentials()

        filename = url.split('/')[-1]
        if not quiet:
            print('{0}/{1}: {2}'.format(str(index).zfill(len(str(url_count))),
                                        url_count, filename))

        try:
            response = get_login_response(url, credentials, token)
            length = int(response.headers['content-length'])
            try:
                if not force and length == os.path.getsize(filename):
                    if not quiet:
                        print('  File exists, skipping')
                    continue
            except OSError:
                pass
            count = 0
            chunk_size = min(max(length, 1), 1024 * 1024)
            max_chunks = int(math.ceil(length / chunk_size))
            time_initial = time.time()
            with open(filename, 'wb') as out_file:
                for data in cmr_read_in_chunks(response, chunk_size=chunk_size):
                    out_file.write(data)
                    if not quiet:
                        count = count + 1
                        time_elapsed = time.time() - time_initial
                        download_speed = get_speed(time_elapsed, count * chunk_size)
                        output_progress(count, max_chunks, status=download_speed)
            if not quiet:
                print()
        except HTTPError as e:
            print('HTTP error {0}, {1}'.format(e.code, e.reason))
        except URLError as e:
            print('URL error: {0}'.format(e.reason))
        except IOError:
            raise


def cmr_filter_urls(search_results):
    """Select only the desired data files from CMR response."""
    if 'feed' not in search_results or 'entry' not in search_results['feed']:
        return []

    entries = [e['links']
               for e in search_results['feed']['entry']
               if 'links' in e]
    # Flatten "entries" to a simple list of links
    links = list(itertools.chain(*entries))

    urls = []
    unique_filenames = set()
    for link in links:
        if 'href' not in link:
            # Exclude links with nothing to download
            continue
        if 'inherited' in link and link['inherited'] is True:
            # Why are we excluding these links?
            continue
        if 'rel' in link and 'data#' not in link['rel']:
            # Exclude links which are not classified by CMR as "data" or "metadata"
            continue

        if 'title' in link and 'opendap' in link['title'].lower():
            # Exclude OPeNDAP links--they are responsible for many duplicates
            # This is a hack; when the metadata is updated to properly identify
            # non-datapool links, we should be able to do this in a non-hack way
            continue

        filename = link['href'].split('/')[-1]
        if filename in unique_filenames:
            # Exclude links with duplicate filenames (they would overwrite)
            continue
        unique_filenames.add(filename)

        urls.append(link['href'])

    return urls


def cmr_search(short_name, version, time_start, time_end,
               bounding_box='', polygon='', filename_filter='', quiet=False):
    """Perform a scrolling CMR query for files matching input criteria."""
    cmr_query_url = build_cmr_query_url(short_name=short_name, version=version,
                                        time_start=time_start, time_end=time_end,
                                        bounding_box=bounding_box,
                                        polygon=polygon, filename_filter=filename_filter)
    if not quiet:
        print('Querying for data:\n\t{0}\n'.format(cmr_query_url))

    cmr_scroll_id = None
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE

    urls = []
    hits = 0
    while True:
        req = Request(cmr_query_url)
        if cmr_scroll_id:
            req.add_header('cmr-scroll-id', cmr_scroll_id)
        try:
            response = urlopen(req, context=ctx)
        except Exception as e:
            print('Error: ' + str(e))
            sys.exit(1)
        if not cmr_scroll_id:
            # Python 2 and 3 have different case for the http headers
            headers = {k.lower(): v for k, v in dict(response.info()).items()}
            cmr_scroll_id = headers['cmr-scroll-id']
            hits = int(headers['cmr-hits'])
            if not quiet:
                if hits > 0:
                    print('Found {0} matches.'.format(hits))
                else:
                    print('Found no matches.')
        search_page = response.read()
        search_page = json.loads(search_page.decode('utf-8'))
        url_scroll_results = cmr_filter_urls(search_page)
        if not url_scroll_results:
            break
        if not quiet and hits > CMR_PAGE_SIZE:
            print('.', end='')
            sys.stdout.flush()
        urls += url_scroll_results

    if not quiet and hits > CMR_PAGE_SIZE:
        print()
    return urls


def main(argv=None):
    global short_name, version, time_start, time_end, bounding_box, \
        polygon, filename_filter, url_list

    if argv is None:
        argv = sys.argv[1:]

    force = False
    quiet = False
    usage = 'usage: nsidc-download_***.py [--help, -h] [--force, -f] [--quiet, -q]'

    # try:
    #     opts, args = getopt.getopt(argv, 'hfq', ['help', 'force', 'quiet'])
    #     for opt, _arg in opts:
    #         if opt in ('-f', '--force'):
    #             force = True
    #         elif opt in ('-q', '--quiet'):
    #             quiet = True
    #         elif opt in ('-h', '--help'):
    #             print(usage)
    #             sys.exit(0)
    # except getopt.GetoptError as e:
    #     print(e.args[0])
    #     print(usage)
    #     sys.exit(1)

    # Supply some default search parameters, just for testing purposes.
    # These are only used if the parameters aren't filled in up above.
    if 'short_name' in short_name:
        short_name = 'ATL06'
        version = '003'
        time_start = '2018-10-14T00:00:00Z'
        time_end = '2021-01-08T21:48:13Z'
        bounding_box = ''
        polygon = ''
        filename_filter = '*ATL06_2020111121*'
        url_list = []

    try:
        if not url_list:
            url_list = cmr_search(short_name, version, time_start, time_end,
                                  bounding_box=bounding_box, polygon=polygon,
                                  filename_filter=filename_filter, quiet=quiet)

        cmr_download(url_list, force=force, quiet=quiet)
    except KeyboardInterrupt:
        quit()


import sys
import os
from os.path import basename, exists, expanduser, join
import shutil
import asf_search as asf
import pandas as pd
import xarray as xr
import rioxarray as rxa
from rioxarray.merge import merge_arrays
import shapely.geometry
from datetime import date
from tqdm import tqdm
from glob import glob

import sys
from os.path import expanduser
sys.path.append(expanduser('~/Documents/spicy-snow'))
from spicy_snow.utils.nsidc import cmr_download, cmr_search

import logging
log = logging.getLogger(__name__)

def download_dem(out_dir = './tmp'):
    """
    Function to download snow-free DEMs to outdir
    https://nsidc.org/data/snex20_qsi_dem/versions/1

    Args:
    out_dir: directory to save lidar veg height tifs to

    Returns:
    None
    """
    os.makedirs(out_dir, exist_ok = True)
    old_cd = os.getcwd()
    os.chdir(out_dir)

    short_name = 'SNEX20_QSI_DEM'
    version = '1'
    time_start = '2021-09-16T00:00:00Z'
    time_end = '2021-09-21T23:59:59Z'
    bounding_box = ''
    polygon = ''
    filename_filter = '*_USIDBS_*'
    url_list = []
    
    quiet = False
    force = False

    try:
        if not url_list:
            url_list = cmr_search(short_name, version, time_start, time_end,
                                    bounding_box=bounding_box, polygon=polygon,
                                    filename_filter=filename_filter, quiet=quiet)

        cmr_download(url_list, force=force, quiet=quiet)
    except KeyboardInterrupt:
        quit()

    os.chdir(old_cd)

def download_snow_depth(out_dir = './tmp'):
    """
    Function to download snow depth files to outdir.
    https://nsidc.org/data/snex20_qsi_sd/versions/1

    Args:
    out_dir: directory to save lidar veg height tifs to

    Returns:
    None
    """

    os.makedirs(out_dir, exist_ok = True)
    old_cd = os.getcwd()
    os.chdir(out_dir)

    short_name = 'SNEX20_QSI_SD'
    version = '1'
    time_start = '2020-02-09T00:00:00Z'
    time_end = '2021-03-20T23:59:59Z'
    bounding_box = ''
    polygon = ''
    filename_filter = '*_USIDBS_*'
    url_list = []
    
    quiet = False
    force = False

    try:
        if not url_list:
            url_list = cmr_search(short_name, version, time_start, time_end,
                                    bounding_box=bounding_box, polygon=polygon,
                                    filename_filter=filename_filter, quiet=quiet)

        cmr_download(url_list, force=force, quiet=quiet)
    except KeyboardInterrupt:
        quit()
    
    os.chdir(old_cd)

def download_veg_height(out_dir = './tmp'):
    """
    Function to download vegation height files to outdir
    https://nsidc.org/data/snex20_qsi_vh/versions/1

    Args:
    out_dir: directory to save lidar veg height tifs to

    Returns:
    None
    """

    os.makedirs(out_dir, exist_ok = True)
    old_cd = os.getcwd()
    os.chdir(out_dir)

    short_name = 'SNEX20_QSI_VH'
    version = '1'
    time_start = '2020-02-01T00:00:00Z'
    time_end = '2021-03-20T23:59:59Z'
    bounding_box = '-116.81,39.69,-105.64,44.38'
    polygon = ''
    filename_filter = '*_USIDBS_*'
    url_list = []
    
    quiet = False
    force = False

    try:
        if not url_list:
            url_list = cmr_search(short_name, version, time_start, time_end,
                                    bounding_box=bounding_box, polygon=polygon,
                                    filename_filter=filename_filter, quiet=quiet)

        cmr_download(url_list, force=force, quiet=quiet)
    except KeyboardInterrupt:
        quit()
    
    os.chdir(old_cd)

def make_site_ds(site: str, lidar_dir: str) -> xr.Dataset:
    """
    Makes a dataset of snow depth, veg height, and DEM for a specific site abbreviation
    in the lidar directory. Returns it reprojected to EPSG4326.

    Args:
    site: Site abbreviation to search for
    lidar_dir: Direction of lidar tiffs

    Returns:
    dataset: xarray dataset of dem, sd, vh for site
    """

    dataset = xr.Dataset()

    for img_type in ['SD', 'VH', 'DEM']:
        files = glob(join(lidar_dir, f'*_{img_type}_*_{site}_*.tif'))
        assert files, f"No files found for {img_type} at {site}"
        
        imgs = []
        for f in files:

            date = pd.to_datetime(basename(f).split('_')[-2])

            img = rxa.open_rasterio(f)

            img = img.squeeze(dim = 'band')
            
            img = img.expand_dims(time = [date])

            imgs.append(img)
    
        dataset['lidar-' + img_type.lower()] = xr.concat(imgs, dim = 'time')

    dataset = dataset.rio.reproject('EPSG:4326')
    
    return dataset


from pathlib import Path
import shapely


# Add main repo to path if you haven't added with conda-develop
# import sys
# sys.path.append('path/to/the/spicy-snow/')

from spicy_snow.retrieval import retrieve_snow_depth
from spicy_snow.IO.user_dates import get_input_dates

# change to your minimum longitude, min lat, max long, max lat
area = shapely.geometry.box(-113.2, 43, -113, 43.4)

# this will be where your results are saved
out_nc = Path('~/data/crrel/spicy-test/test.nc').expanduser()

# this will generate a tuple of dates from the previous August 1st to this date
dates = get_input_dates('2021-04-01') # run on all s1 images from (2020-08-01, 2021-04-01) in this example

spicy_ds = retrieve_snow_depth(area = area, dates = dates, 
                               work_dir = Path('~/data/crrel/spicy-test/').expanduser(), 
                               job_name = f'testing_spicy',
                               existing_job_name = 'testing_spicy',
                               debug=False,
                               outfp=out_nc)



"""
Functions to download PROBA-V forest-cover-fraction images for specific geometries
"""

import sys
import os
from os.path import basename, exists, expanduser, join
import shutil
import shapely
import rioxarray as rxa
import xarray as xr

import logging
log = logging.getLogger(__name__)

sys.path.append(expanduser('~/Documents/spicy-snow'))
from spicy_snow.utils.download import url_download


def download_fcf(dataset: xr.Dataset, out_fp: str, environment: dict = None) -> xr.Dataset:
    """
    Download PROBA-V forest-cover-fraction images.

    Args:
    dataset: large dataset to add IMS data to
    out_fp: filepath to save tiff of results

    Returns:
    dataset: large dataset with 'fcf' added as data variable
    """
    log.debug("Downloading Forest Cover")
    # this is the url from Lievens et al. 2021 paper
    fcf_url = 'https://zenodo.org/record/3939050/files/PROBAV_LC100_global_v3.0.1_2019-nrt_Tree-CoverFraction-layer_EPSG-4326.tif'
    # download just forest cover fraction to out file
    url_download(fcf_url, out_fp)
    # open as dataArray and return
    fcf = rxa.open_rasterio(out_fp)

    # reproject FCF and clip to match dataset
    log.debug(f"Clipping FCF to {dataset['s1'].rio.bounds()}")
    # clip first to avoid super long reproject processes
    fcf = fcf.rio.clip_box(*dataset['s1'].rio.bounds())
    # reproject FCF to match dataset
    fcf = fcf.rio.reproject_match(dataset['s1'])
    # remove band dimension as it only has one band
    fcf = fcf.squeeze('band')
    # if max is greater than 1 set to 0-1
    if fcf.max() >= 1:
        log.debug("fcf max > 1 so dividing by 100")
        fcf = fcf / 100
        log.debug(f"New fcf max is {fcf.max()} and min is {fcf.min()}")
    
    assert fcf.max() <= 1, "Forest cover fraction must be bounded 0-1"
    assert fcf.min() >= 0, "Forest cover fraction must be bounded 0-1"

    log.debug(f'FCF min: {fcf.min()}')
    log.debug(f'FCF max: {fcf.max()}')
    log.debug(f'FCF mean: {fcf.mean()}')
    
    # merge FCF and name it 'fcf' as a data variable
    dataset = xr.merge([dataset, fcf.rename('fcf')])

    dataset.to_netcdf(f'{environment['workdir']}/tmp/sentinel1_snow_fcf.nc')

    return dataset

# End of file


"""
Functions to identify, mask, and create weights for wet-snow.
"""
import sys
import numpy as np
import xarray as xr
from typing import Union

import logging
log = logging.getLogger(__name__)

def id_newly_wet_snow(dataset: xr.Dataset, wet_thresh: int = -2, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Identifies time steps with newly wet snow. Identifies time slices where
    deltaVV decreases by 2dB in pixels with FCF > 0.5 and deltaCR decreases by
    2dB in pixels with FCF < 0.5.

    Args:
    dataset: xarray dataset with deltaVV and deltaCR as data vars
    wet_thresh: decrease in dB of deltaVV or deltaCR to identify melting
    inplace: return copy of dataset or operate on dataset inplace?

    Returns:
    dataset: xarray data with melting data var
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)
    
    # check we have the neccessary variables
    necessary_vars = set(['fcf', 'deltaCR', 'deltaVV'])
    assert necessary_vars.issubset(set(dataset.data_vars)),\
          f"Missing variables {necessary_vars.difference(set(dataset.data_vars))}"
    
    # add wet_flag to dataset if not already present    
    if 'wet_flag' not in dataset.data_vars:
        dataset['wet_flag'] = xr.zeros_like(dataset['deltaVV'])

    # identify possible newly wet snow in regions FCF < 0.5 with deltaCR
    dataset['wet_flag'] = dataset['wet_flag'].where(((dataset['fcf'] > 0.5) | (dataset['deltaCR'] > wet_thresh) | (dataset['deltaCR'].isnull())), 1)
    # identify possible newly wet snow in regions FCF > 0.5 with deltaVV
    dataset['wet_flag'] = dataset['wet_flag'].where(((dataset['fcf'] < 0.5) | (dataset['deltaVV'] > wet_thresh) | (dataset['deltaVV'].isnull())), 1)

    # mask nans from Sentinel-1 data
    dataset['wet_flag'] = dataset['wet_flag'].where(~dataset['s1'].sel(band = 'VV').isnull(),np.nan)
    
    if not inplace:
        return dataset

def id_newly_frozen_snow(dataset: xr.Dataset, freeze_thresh: int = 2, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Identifies time steps with probable re-frozen snow. Identifies time slices where
    deltaGammaNaught increases by 2 dB

    Args:
    dataset: xarray dataset with deltaGammaNaught as data var
    freeze_thresh: increase in dB of deltaGammaNaught to identify refreeze
    inplace: return copy of dataset or operate on dataset inplace?

    Returns:
    dataset: xarray data with freezing data var
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)

    # check we have the neccessary variables
    necessary_vars = set(['deltaGamma'])
    assert necessary_vars.issubset(set(dataset.data_vars)),\
          f"Missing variables {necessary_vars.difference(set(dataset.data_vars))}"

    # add wet_flag to dataset if not already present    
    if 'freeze_flag' not in dataset.data_vars:
        dataset['freeze_flag'] = xr.zeros_like(dataset['deltaVV'])

    # identify possible re-freezing by increases of deltaGammaNaught of 2dB
    dataset['freeze_flag'] = dataset['freeze_flag'].where((dataset['deltaGamma'] < freeze_thresh) | (dataset['deltaGamma'].isnull()), 1)

    # mask nans from Sentinel-1 data
    dataset['freeze_flag'] = dataset['freeze_flag'].where(~dataset['snow_index'].isnull())

    if not inplace:
        return dataset

def id_wet_negative_si(dataset: xr.Dataset, wet_SI_thresh = 0, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Additional wet snow criteria if sd retrieval (snow-index since they are linear)
    becomes negative with snow cover is present we set pixel to wet.

    Args:
    dataset: xarray dataset with snow_index as data vars
    wet_SI_thresh: threshold to use for negative SI [default: 0]
    inplace: return copy of dataset or operate on dataset inplace?

    Returns:
    dataset: xarray data with wet_snow data var
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)

    # check we have the neccessary variables
    necessary_vars = set(['snow_index', 'ims'])
    assert necessary_vars.issubset(set(dataset.data_vars)),\
          f"Missing variables {necessary_vars.difference(set(dataset.data_vars))}"

    # add alt_wet_flag to dataset if not already present    
    if 'alt_wet_flag' not in dataset.data_vars:
        dataset['alt_wet_flag'] = xr.zeros_like(dataset['deltaVV'])

    # identify wetting of snow by negative snow index with snow present
    dataset['alt_wet_flag'] = dataset['alt_wet_flag'].where(((dataset['ims'] != 4) | (dataset['snow_index'] > wet_SI_thresh) | (dataset['snow_index'].isnull())), 1)

    # mask nans from Sentinel-1 data
    dataset['alt_wet_flag'] = dataset['alt_wet_flag'].where(~dataset['snow_index'].isnull())

    if not inplace:
        return dataset

def flag_wet_snow(dataset: xr.Dataset, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Identifies time steps with wet snow. Sets all time slices, for a relative orbit,
    as dry until the first melting then sets all time steps, for that relative orbit,
    as wet until either end of time series or next re-freezing.

    If re-freezing revert to set all as dry until next melt. Repeat to end of time series.
    
    If after Feburary 1st until August 1st if last two of four relative orbits
    were classified as wet we set remained to wet to stop retrievals

    Args:
    dataset: xarray dataset with melting, freezing as data vars
    inplace: return copy of dataset or operate on dataset inplace?

    Returns:
    dataset: xarray data with wet_snow data var
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)

    # check we have the neccessary variables
    necessary_vars = set(['wet_flag', 'alt_wet_flag', 'freeze_flag'])
    assert necessary_vars.issubset(set(dataset.data_vars)),\
          f"Missing variables {necessary_vars.difference(set(dataset.data_vars))}"
    
    # create new data variable wet_snow and set to 0
    dataset['wet_snow'] = xr.zeros_like(dataset['wet_flag'])
    dataset['perma_wet'] = xr.zeros_like(dataset['wet_flag'])

    # get all unique relative orbits
    orbits = np.unique(dataset['relative_orbit'].values)

    # Identify previous image from the same relative orbit (6, 12, 18, or 24 days ago)
    for orbit in orbits:
        # select just that orbit to do wet snow propogation
        orbit_dataset = dataset.sel(time = dataset.relative_orbit == orbit)

        prev_time = None

        # loop through time steps in this orbit
        for ts in orbit_dataset.time:

            # if not first round then pull in previous time step's values for wet-snow
            if prev_time:
                dataset['wet_snow'].loc[dict(time = ts)] = dataset.sel(time = prev_time)['wet_snow']

            # add newly wet snow flags to old wet snow and then bound at 1

            dataset['wet_snow'].loc[dict(time = ts)] = xr.where(~dataset.sel(time = ts)['wet_flag'].isnull(),dataset.sel(time = ts)['wet_snow'] + dataset.sel(time = ts)['wet_flag'],np.nan)
            dataset['wet_snow'].loc[dict(time = ts)] = xr.where(~dataset.sel(time = ts)['alt_wet_flag'].isnull(),dataset.sel(time = ts)['wet_snow'] + dataset.sel(time = ts)['alt_wet_flag'],np.nan)
            dataset['wet_snow'].loc[dict(time = ts)] = dataset.sel(time = ts)['wet_snow'].where((dataset.sel(time = ts)['wet_snow'] < 1) | (dataset.sel(time = ts)['wet_snow'].isnull()), 1)

            # add newly frozen snow flags to old wet snow and then bound at 0 to avoid negatives
            dataset['wet_snow'].loc[dict(time = ts)] = xr.where(~dataset.sel(time = ts)['freeze_flag'].isnull(),dataset.sel(time = ts)['wet_snow'] - dataset.sel(time = ts)['freeze_flag'],np.nan)
            dataset['wet_snow'].loc[dict(time = ts)] = dataset.sel(time = ts)['wet_snow'].where((dataset.sel(time = ts)['wet_snow'] > 0) | (dataset.sel(time = ts)['wet_snow'].isnull()), 0)

            # set non snow (IMS != 4) to not wet (0)
            dataset['wet_snow'].loc[dict(time = ts)] = dataset.sel(time = ts)['wet_snow'].where(dataset.sel(time = ts)['ims'] == 4, 0)

            # make nans at areas without S1 data
            dataset['wet_snow'].loc[dict(time = ts)] = dataset.sel(time = ts)['wet_snow'].where(~dataset['s1'].sel(time = ts, band = 'VV').isnull(), np.nan)
                    
            prev_time = ts

        # if >50% wet of last 4 cycles after feb 1 then set remainer till
        # august 1st to perma-wet(2?)
    
        melt_season = (dataset['time.month'] > 1) & (dataset['time.month'] < 8)

        # get images of this relative orbit and in the melt season
        melt_season = (dataset['time.month'] > 1) & (dataset['time.month'] < 8)

        # get images of this relative orbit and in the melt season
        melt_orbit = (melt_season & (dataset.relative_orbit == orbit))

        # check if there are at least 4 time slices in melt season for this orbit
        if len(dataset['perma_wet'].loc[dict(time = melt_orbit)]) < 4:
            continue

        # first set all perma wet to match the flagged wet times from dB drop
        dataset['perma_wet'].loc[dict(time = melt_orbit)] = \
            dataset['wet_flag'].loc[dict(time = melt_orbit)]
        
        # then set times that were made wet by negative snow index to wet as well
        dataset['perma_wet'].loc[dict(time = melt_orbit)] = dataset['perma_wet'].loc[dict(time = melt_orbit)] + \
            dataset['alt_wet_flag'].loc[dict(time = melt_orbit)]
        
        # now if we are over 1 (ie it was flagged wet by dB and negative SI flags) we should floor those back to 1
        dataset['perma_wet'].loc[dict(time = melt_orbit)] = \
            dataset['perma_wet'].loc[dict(time = melt_orbit)].where((dataset['perma_wet'].loc[dict(time = melt_orbit)] <= 1) | (dataset['perma_wet'].loc[dict(time = melt_orbit)].isnull()) , 1)
        
        # now calculate the rolling mean of the perma wet so we have a % 0-1 of days out of 4 that were flagged
        dataset['perma_wet'].loc[dict(time = melt_orbit)] = \
            dataset['perma_wet'].loc[dict(time = melt_orbit)].rolling(time = 4).mean()
        
        # then propogate forward so that if we get to > 50% in a 4 image window we mask the remained of the melt season
        # this will fail if bottleneck is installed due to it lacking the min_periods keyword
        # see: https://github.com/pydata/xarray/issues/4922


        if 'bottleneck' not in sys.modules:
            dataset['perma_wet'].loc[dict(time = melt_orbit)] = \
                dataset['perma_wet'].loc[dict(time = melt_orbit)].rolling(time = len(orbit_dataset.time), min_periods = 1).max()
        else:
            log.info("bottleneck installed. Consider pip uninstalling and re-running if this fails.")
            dataset['perma_wet'].loc[dict(time = melt_orbit)] = \
                dataset['perma_wet'].loc[dict(time = melt_orbit)].rolling(time = len(orbit_dataset.time)).max()

        # set perma wet to nans if no S1 data
        dataset['perma_wet'].loc[dict(time = melt_orbit)] = dataset.sel(dict(time = melt_orbit))['perma_wet'].where(~dataset['s1'].sel(dict(time = melt_orbit, band = 'VV')).isnull(), np.nan)

        # set perma wet to 0 if no snow in IMS
        dataset['perma_wet'].loc[dict(time = melt_orbit)] = dataset.sel(dict(time = melt_orbit))['perma_wet'].where(dataset['ims'].sel(dict(time = melt_orbit)) == 4, 0)

    # if we have no data just set it to not be flagged perma_wet
    dataset['perma_wet'] = dataset['perma_wet'].where(~dataset['perma_wet'].isnull(), 0)

    # if less than 50% are wet then keep the save value for wet_snow otherwise set to 1
    dataset['wet_snow'] = dataset['wet_snow'].where(dataset['perma_wet'] < 0.5, 1)

    dataset['wet_snow'].loc[dict(time = ts)] = dataset.sel(time = ts)['wet_snow'].where(dataset.sel(time = ts)['ims'] == 4, 0)

    return dataset



import numpy as np
import pandas as pd
import xarray as xr

from pathlib import Path

from tqdm import tqdm
print('starting')

in_dir = Path('~/scratch/spicy/SnowEx-Data/').expanduser().resolve()
in_dir = Path('~/spicy-snow/SnowEx-Data-Hans/').expanduser().resolve()

out_dir = Path('~/scratch/spicy/SnowEx-Hans-s1').expanduser()

dss = {fp.stem: xr.open_dataset(fp) for fp in in_dir.glob('*.nc')}

# data_dir = Path('~/scratch/West_US_Canada').expanduser()
# # assert data_dir.exists()
# fps = list(data_dir.glob('snd_*.nc'))

# for stem, old_ds in dss.items():
#     new_dss = []
#     print(stem)
#     for fp in tqdm(fps):
#         dt = pd.to_datetime(fp.stem.split('_')[1])
#         if dt > old_ds.time[0] and dt < old_ds.time[-1]:
#             ds = xr.open_dataset(fp)['snd']
#             ds = ds.expand_dims(time = [dt])
#             ds = ds.sel(lat = slice(old_ds.y.max(), old_ds.y.min()), lon = slice(old_ds.x.min(), old_ds.x.max()))
#             new_dss.append(ds)

#     full = xr.concat(new_dss, dim = 'time')
#     full = full.dropna(dim ='time', how ='all').sortby('time')
#     full = full.rename({'lat':'y'})
#     full = full.rename({'lon':'x'})

#     full.to_netcdf(out_dir.joinpath(stem+'.nc'))

# experiemential data
data_dir = Path('/bsuhome/zacharykeskinen/scratch/experimental_sentinel1_snow_depth_data')
out_dir = Path('~/scratch/spicy/experimental_SnowEx-Hans-s1').expanduser()
fps = list(data_dir.glob('*.nc'))

# gives us the lats and longitudes
old = xr.open_dataset('~/scratch/SD_20180126.nc')

for stem, old_ds in dss.items():
    new_dss = []
    print(stem)
    for fp in fps:
        dt = pd.to_datetime(fp.stem.split('_')[1])
        if dt > pd.to_datetime('2019-08-01') and dt < pd.to_datetime('2021-04-01'):
            ds = xr.open_dataset(fp)['snd']
            ds = ds.isel(ease2_x = slice(0, 34703))
            ds = xr.DataArray(ds.data, dims = ['lon','lat'], coords = {'lon': ('lon', old.lon.data), 'lat': ('lat', old.lat.data[::-1])})
            ds = ds.expand_dims(time = [dt])
            ds = ds.sel(lat = slice(old_ds.y.max(), old_ds.y.min()), lon = slice(old_ds.x.min(), old_ds.x.max()))
            new_dss.append(ds)

    full = xr.concat(new_dss, dim = 'time')
    full = full.dropna(dim ='time', how ='all').sortby('time')
    full = full.rename({'lat':'y'})
    full = full.rename({'lon':'x'})

    full.to_netcdf(out_dir.joinpath(stem+'.nc'))


from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser
import geopandas as gpd
import pandas as pd
from pathlib import Path
from asf_snow.utils.analyze_sentinel1_snow import draw_combined_gdf, draw_scatter_plot

file = '/home/jiangzhu/data/crrel/sentinel1_snowEx21_ts_snow_pits_v1/fraser/fraser_202011_202105/sentinel1_202011_202105_500_12h_12h_average_y_fraser_experimental_forest.shp'
def single_statistic_results(file: str, timethresh=None):

    lst = Path(file).stem.split("_")
    res = int(lst[3])
    timeint = int(lst[4][:-1])
    timethresh = int(lst[5][:-1])
    method = lst[6]
    precip = lst[7]
    outfile = f'/home/jiangzhu/data/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/sentinel1_202011_202104_{res}_{timeint}h_{timethresh}h_{method}_{precip}_Fraser.png'

    gdf = gpd.read_file(file)


    draw_combined_gdf(gdf, res=res, timeint=timeint, timethresh=timethresh, flg=method, outfile=outfile, precip=precip)


    # draw_scatter_plot(tot, res=res)

    # calculate the correlation between snow depth and observation


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--file', type=str,  required=True)

    args = parser.parse_args()

    single_statistic_results(file)

    print("completed ...")


import numpy as np
import pandas as pd
import xarray as xr
import matplotlib.pyplot as plt

from multiprocessing import Pool
from functools import partial

from pathlib import Path
from tqdm import tqdm

from scipy.stats import pearsonr
from sklearn.metrics import mean_absolute_error as mae
from sklearn.metrics import mean_squared_error

from itertools import product
import warnings

# from tqdm.contrib.itertools import product

def get_stats(a, b):
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", message="An input array is constant")
        r, p = pearsonr(a, b)
    error = mae(a, b)
    rmse = mean_squared_error(a, b, squared=False)
    bias = np.mean(a - b)
    return r, error, rmse, bias


def make_stat_da(tree_idx_low, tree_idx_high, loc_fp):
    
    print(f'Starting {loc_fp.stem}...')

    stems= [f.stem for f in list(loc_fp.glob('*_*_*.npy'))]

    A = np.unique([float(s.split('_')[0]) for s in stems])
    B = np.unique([float(s.split('_')[1]) for s in stems])
    C = np.unique([float(s.split('_')[2]) for s in stems])

    res = np.zeros((1, len(A), len(B), len(C)))

    da = xr.DataArray(res, coords = [[loc_fp.stem], A, B, C], dims = ['location', 'A', 'B','C'], name = 'pearsonr')
    res_ds = xr.merge([da, da.copy().rename('mae'), da.copy().rename('rmse'), da.copy().rename('bias')])

    lidar_orig = np.load(loc_fp.joinpath('lidar.npy'))
    elev = np.load(loc_fp.joinpath('elev.npy'))
    trees = np.load(loc_fp.joinpath('trees.npy'))

    idx = (trees <= tree_idx_high) & (elev > 0) & (trees >= tree_idx_low)

    if 'Cottonwood' in loc_fp.stem:
        print(f"Size of datarray: {res_ds['pearsonr'].data.shape}")
        print(f'Used fraction: {np.sum(idx)/elev.size}')

    for a, b, c in product(A, B, C):
        sds_orig = np.load(loc_fp.joinpath(f'{a}_{b}_{c}.npy'))
        combo = np.vstack([lidar_orig, sds_orig])
        combo = combo.T[idx].T

        sds, lidar = combo
        r, mean_error, rmse, bias = get_stats(lidar, sds)
        res_ds['pearsonr'].loc[dict(location = loc_fp.stem, A = a, B = b, C = c)] = r
        res_ds['mae'].loc[dict(location = loc_fp.stem, A = a, B = b, C = c)] = mean_error
        res_ds['rmse'].loc[dict(location = loc_fp.stem, A = a, B = b, C = c)] = rmse
        res_ds['bias'].loc[dict(location = loc_fp.stem, A = a, B = b, C = c)] = bias
    
    for dv in res_ds.data_vars:
        res_ds[dv] = res_ds[dv].astype(float)
    res_ds.to_netcdf(loc_fp.parent.joinpath('stats_ncs', loc_fp.stem + f'_{tree_idx_low:.2f}_{tree_idx_high:.2f}_stats.nc'))

    print(f'Finishing {loc_fp.stem}!')


param_fp = Path('/bsuhome/zacharykeskinen/scratch/spicy/param_npys')

param_fp.joinpath('stats_ncs').mkdir(exist_ok = True)


out_fp = Path('/bsuhome/zacharykeskinen/scratch/spicy/param_stats.nc')

if out_fp.exists():
    print('Already exists...')

pool = Pool()
    
pool.map(partial(make_stat_da, 0, 1), param_fp.glob('*_*-*-*'))
das = [xr.open_dataset(fp) for fp in param_fp.joinpath('stats_ncs').glob('*0.00_1.00_stats.nc')]
ds = xr.merge(das)
ds.to_netcdf(param_fp.joinpath('param_stats_all.nc'))

pool = Pool()
pool.map(partial(make_stat_da, 0, 0.25), param_fp.glob('*_*-*-*'))
das = [xr.open_dataset(fp) for fp in param_fp.joinpath('stats_ncs').glob('*0.00_0.25_stats.nc')]
ds = xr.merge(das)
ds.to_netcdf(param_fp.joinpath('param_stats_low_fcf.nc'))

pool = Pool()
pool.map(partial(make_stat_da, 0.75, 1), param_fp.glob('*_*-*-*'))
das = [xr.open_dataset(fp) for fp in param_fp.joinpath('stats_ncs').glob('*0.75_1.00_stats.nc')]
ds = xr.merge(das)
ds.to_netcdf(param_fp.joinpath('param_stats_high_fcf.nc'))

# locs = list(param_fp.glob('*'))
# locs = [l.stem for l in locs]


from asf_snow.utils.combine_multiple_statistic_results import *

from asf_snow.utils.analyze_sentinel1_snow import *

# file = "/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/sentinel1_202011_202104_500_36h_6h_snex_close_y_boise_river_basin.geojson"

# file = "/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/sentinel1_202011_202104_500_12h_12h_snex_average_y_boise_river_basin.geojson"


def extract_arguments(file):
    out_dir = str(Path(file).parent.parent)
    lst = Path(file).stem.split("_")
    if 'fairbanks' in file or 'northslope' in file:
        lst = lst[1:]

    res = int(lst[3])
    timeint = int(lst[4][:-1])
    timethresh = int(lst[5][:-1])
    base = lst[6]
    method = lst[7]
    precip = lst[8]
    loc = lst[9]

    return out_dir, res, timeint, timethresh, base, method, precip, loc


def get_gdf_with_unique_pixels(gdf):
    gdf_o = gdf.copy(deep=True)
    gdf = gdf.drop_duplicates('s1geometry')
    gdf = gdf.reset_index(drop=True)

    # Average the snow depths of the duplicated records in gdf_o
    s1geometry = gdf['s1geometry']
    for index, item in s1geometry.items():
        snex_sd_mean = gdf_o[gdf_o['s1geometry'] == item]['snex_sd'].mean()
        gdf.loc[gdf['s1geometry'] == item, ['snex_sd']] = snex_sd_mean
    return gdf


def draw_diff_error_count(file):

    gdf = gpd.read_file(file)

    if 'fairbanks' in Path(file).stem:
        gdf = get_gdf_with_unique_pixels(gdf)

    out_dir, res, timeint, timethresh, base, method, precip, loc = extract_arguments(file)

    # plot_type = 'combined'

    # outfile = f'{out_dir}/{plot_type}_{res}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_{loc}.png'

    # draw_combined_gdf(gdf, res, timeint, timethresh, base, method, outfile, precip)

    plot_type = 'diff_error_count'

    outfile = f'{out_dir}/{plot_type}_{res}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_{loc}.png'

    # snex_ts,
    # draw_diff_error_count_plot(gdf, res, timeint, timethresh, base, method, outfile, precip)

    # snex23, alaska
    draw_diff_error_count_plot_index(gdf, res, timeint, timethresh, base, method, outfile, precip)

    plot_type = 'histogram'

    outfile = f'{out_dir}/{plot_type}_{res}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_{loc}.png'

    draw_histgram(gdf, res, timeint, timethresh, base, method, outfile, precip)


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--file', type=str,  required=True)

    args = parser.parse_args()

    draw_diff_error_count(args.file)

    print("completed ...")






import shapely
from pathlib import Path
import numpy as np
import pandas as pd
import xarray as xr
import rioxarray as rxa
from rasterio.enums import Resampling

from datetime import datetime

# from asf_snow.spicy_snow.retrieval import retrieve_snow_depth
from asf_snow.retrieval_sentinel1_snow import retrieve_snow_depth

from spicy_snow.download.snowex_lidar import download_dem, download_snow_depth,\
      download_veg_height

from asf_snow.spicy_snow.IO.user_dates import get_input_dates


lidar_dir = Path('/home/jiangzhu/data/crrel/lidar_via_zach')

sites = {'USCOCP': 'Cameron', 'USCOFR': 'Frasier', 'USIDBS': 'Banner', 
         'USIDDC': 'Dry_Creek', 'USIDMC': 'Mores', 'USUTLC': 'Little_Cottonwood'}

sites = {'USIDBS': 'Banner'}

filename_filter = '*20200218_20200219*'

def make_site_ds(site: str, lidar_dir: Path, filename_filter='*') -> xr.Dataset:
    """
    Makes a dataset of snow depth, veg height, and DEM for a specific site abbreviation
    in the lidar directory. Returns it reprojected to EPSG4326.

    Args:
    site: Site abbreviation to search for
    lidar_dir: Direction of lidar tiffs

    Returns:
    dataset: xarray dataset of dem, sd, vh for site
    """

    dataset = xr.Dataset()

    for img_type in ['SD', 'VH', 'DEM']:
        # files = glob(join(lidar_dir, f'*_{img_type}_*_{site}_*.tif'))
        if img_type == 'DEM':
            files = lidar_dir.glob(f'*_{img_type}_*_{site}_*.tif')
        else:
            files = lidar_dir.glob(f'*_{img_type}_*_{site}_{filename_filter}.tif')

        assert files, f"No files found for {img_type} at {site}"
        
        imgs = []
        for f in files:
            print(f)

            if img_type != 'DEM':
                date = pd.to_datetime(f.stem.split('_')[-2])
                sd_date = date
            else:
                date = sd_date

            img = rxa.open_rasterio(f)

            img = img.squeeze(dim = 'band')
            
            if img_type != 'DEM':
                img = img.expand_dims(time = [date])

            imgs.append(img)
        
        if img_type != 'DEM':
            dataset['lidar-' + img_type.lower()] = xr.concat(imgs, dim = 'time')
        else:
            dataset['lidar-' + img_type.lower()] = imgs[0]

    dataset = dataset.rio.reproject('EPSG:4326')
    
    return dataset


# param_set = Path('/bsuhome/zacharykeskinen/spicy-analysis/results/params/stat_res_0.5tree.csv')
# param_df = pd.DataFrame()
# if param_set.exists():
#     param_df = pd.read_csv(param_set, index_col = 0)

def main():

    for site, site_name in sites.items():
        print(''.center(40, '-'))
        print(f'Starting {site_name}')
        if Path(f'{lidar_dir}/lidar_ds_site_validdata.nc').is_file():
            lidar_ds_site = xr.open_dataset(f'{lidar_dir}/lidar_ds_site_validdata.nc')
        else:
            if Path(f'{lidar_dir}/lidar_ds_site.nc').is_file():
                lidar_ds_site = xr.open_dataset(f'{lidar_dir}/lidar_ds_site.nc')
            else:
                lidar_ds_site = make_site_ds(site, lidar_dir = lidar_dir, filename_filter=filename_filter)
            # lidar_ds_site = lidar_ds_site.where(lidar_ds_site < 1e9).where(lidar_ds_site >= 0)
            ch = np.logical_and(lidar_ds_site < 1e9, lidar_ds_site >= 0)
            lidar_ds_site['lidar-sd'] = lidar_ds_site['lidar-sd'].where(ch['lidar-sd'])
            lidar_ds_site['lidar-vh'] = lidar_ds_site['lidar-vh'].where(ch['lidar-vh'])
            lidar_ds_site['lidar-dem'] = lidar_ds_site['lidar-dem'].where(ch['lidar-dem'])
            lidar_ds_site.to_netcdf(f'{lidar_dir}/lidar_ds_site_validdata.nc')

        area = shapely.geometry.box(*lidar_ds_site.rio.bounds())

        for date in lidar_ds_site.time:

            # if param_df.size > 0:
            #     out_dir = Path('/bsuhome/zacharykeskinen/scratch/spicy/SnowEx-Data-params/')
            # else:
            out_dir = Path(f'{lidar_dir}/scratch/spicy/SnowEx-Data-lave/')
            out_dir.mkdir(parents=True, exist_ok = True)

            out_nc = out_dir.joinpath(f'{site_name}_{(date).dt.strftime("%Y-%m-%d").values}.nc')

            if out_nc.exists():
                print(f'Outfile {out_nc} exists already.')
                continue

            print(f'Starting {site_name} snow depth @ {date.values}')

            if date.dt.month > 4:
                continue

            lidar_ds = lidar_ds_site.sel(time = date)

            dates = get_input_dates(date.data + pd.Timedelta('14 day'))

            if Path(f'{lidar_dir}/spicy_ds.nc').is_file():
                spicy_ds = xr.open_dataset(f'{lidar_dir}/spicy_ds.nc')
            # if param_df.size > 0:
            else:
                spicy_ds = retrieve_snow_depth(area = area, dates = dates, work_dir = '/home/jiangzhu/data/crrel/lidar_via_zach/scratch/spicy/', \
                                            job_name = f'spicy_{site}_{dates[1]}', existing_job_name = f'spicy_{site}_{dates[1]}', \
                                            params = [1.5, 0.1, 0.59])
                spicy_ds.to_netcdf(f'{lidar_dir}/spicy_ds.nc')

            # else:
                # spicy_ds = retrieve_snow_depth(area = area, dates = dates, work_dir = '/bsuhome/zacharykeskinen/scratch/spicy/', job_name = f'spicy_{site}_{dates[1]}', existing_job_name = f'spicy_{site}_{dates[1]}')

            for var in lidar_ds:
                lidar_ds[var] = lidar_ds[var].rio.write_nodata(np.nan)

            lidar_ds = lidar_ds.rio.reproject_match(spicy_ds, resampling = Resampling.average, nodata=np.nan)

            ds = xr.merge([spicy_ds, lidar_ds], combine_attrs = 'drop_conflicts')

            # ds = ds[['lidar-sd', 'lidar-vh', 'lidar-dem', 'snow_depth', 's1', 'wet_snow']]

            ds.attrs['site'] = site_name
            ds.attrs['site_abbrev'] = site
            ds.attrs['lidar-flight-time'] = str((date[0]).dt.strftime("%Y-%m-%d").values)
            ds.attrs['processing-date'] = f'{datetime.now()}'

            try:
                ds.to_netcdf(out_nc)
            except:
                print('Unable to create netcdf4 for {site_name}')


if __name__ == '__main__':

    main()


"""
Functions to create and set loggers.
"""

import sys
import os
import logging
from os.path import join, basename
from datetime import datetime

def setup_logging(log_dir = './logs/', debug = False):

    os.makedirs(log_dir, exist_ok= True)
    log_prefix = 'spicy-snow'
    timestr = datetime.now().strftime("%Y%m%d-%H%M%S")

    logging.basicConfig(level=logging.INFO,
                        
        format=f"(spicy-snow {__name__} %(levelname)s) %(message)s",
        # saves out to log file and outputs to command line.

        handlers=[
            logging.FileHandler(join(log_dir, f'{log_prefix}_{timestr}.log')),
            logging.StreamHandler(sys.stdout)]
    )
    
    log = logging.getLogger()

    if debug:
        log.setLevel(logging.DEBUG) 


"""
Utility functions for downloading files
"""
import sys
import time
from pathlib import Path
from os.path import basename, exists
from os import stat, remove
import gzip
from urllib.request import urlretrieve, urlopen
import numpy as np
import logging
log = logging.getLogger(__name__)


def reporthook(count, block_size, total_size):
    """
    Hook for urlib downloads to get progress readout.
    """
    global start_time
    if count == 0:
        start_time = time.time()
        return
    duration = time.time() - start_time
    progress_size = int(count * block_size)
    speed = min(int(count*block_size*100/total_size),100)
    percent = int(count * block_size * 100 / total_size)
    sys.stdout.write("\r...%d%%, %d MB, %d KB/s, %d seconds passed" %
                    (percent, progress_size / (1024 * 1024), speed, duration))
    sys.stdout.flush()


def url_download(url, out_fp, overwrite = False, verbose = True):
    """
    Downloads url with a progress bar and overwrite check.
    """
    # get the file size of url

    with urlopen(url) as file:
        meta = file.info()
        fsize = np.int64(meta.get('Content-Length'))

    # check if file already exists
    if not exists(out_fp) or overwrite == True:
        # progress bar for your download?
        if verbose:
            log.info(f'Downloading {basename(out_fp)}.')
            urlretrieve(url, out_fp, reporthook)
            log.info('')
        # or not?
        else:
            urlretrieve(url, out_fp)
    # if already exists, but its size is less than url, download again.
    else:

        if fsize != Path(out_fp).stat().st_size:
            log.info(f'Downloading {basename(out_fp)}.')
            urlretrieve(url, out_fp, reporthook)
        else:
            if verbose:
                log.info(f'{basename(out_fp)} already exists. Skipping.')

    # check file size

    if fsize == Path(out_fp).stat().st_size:
        log.info(f'{basename(out_fp)} is complete.')
        return True
    else:
        return False


def decompress(infile, tofile):
    """
    Decompress gzipped infile to outfile location.
    """

    log.debug(f"decompressing {infile} to {tofile}")

    with open(infile, 'rb') as inf, open(tofile, 'wb') as ouf:
        decom_str = gzip.decompress(inf.read())
        ouf.write(decom_str)

        return tofile